/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "zx.h"
#include "os_interface.h"
#include "zx_driver.h"
#include "zx_ioctl.h"
#include "zx_version.h"
#include "zx_debugfs.h"

char *zx_fb_mode = NULL; //"800x600-32@60";
#ifdef ZX_HW_NULL
int   zx_fb      = 0;
#else
int   zx_fb      = 1;
#endif
int   zx_flip    = 0;

zx_card_t zx_cards[ZX_MAX_CARDS];

#if 0
static int zx_open(struct inode *inode, struct file *filp)
{
    zx_card_t *zx = NULL;

    zx_file_t *priv;

    int minor;

    int err = -ENODEV;

#ifndef MINOR
    minor = minor(inode->i_rdev);
#else
    minor = MINOR(inode->i_rdev);
#endif

    if(!((minor >= 0) && (minor < ZX_MAX_CARDS)))
    {
        return err;
    }

    zx = &zx_cards[minor];

    if(!zx->adapter)
    {
        return err;
    }

    priv = zx_calloc(sizeof(zx_file_t));

    if(!priv)
    {
        return -ENOMEM;
    }

    filp->private_data = priv;

    priv->filp        = filp;
    priv->card        = zx;
    priv->lock        = zx_create_mutex();

    init_waitqueue_head(&priv->wait_queue);

    return 0;
}

static int zx_release(struct inode *inode, struct file *filp)
{
    zx_file_t *priv = filp->private_data;
    zx_card_t *zx  = priv->card;

    if(priv->wait_events)
    {
        zx_remove_wait(zx, priv);
    }

    if(priv->hold_lock)
    {
        zx_mutex_unlock(zx->lock);
    }

    if(priv->gpu_device)
    {
        zx_cleanup_2d_screen(zx, priv);

        if(zx->debugfs_dev)
        {
            zx_debugfs_remove_node(zx->debugfs_dev, priv->gpu_device);
        }

        krnl_final_cleanup(zx->adapter, priv->gpu_device);

        priv->gpu_device = 0;
    }

    zx_destroy_mutex(priv->lock);
    zx_free(priv);
    filp->private_data = NULL;

    return 0;
}

static long zx_unlocked_ioctl(struct file *filp, unsigned int cmd, unsigned long arg)
{
    unsigned int nr   = ZX_IOCTL_NR(cmd);
    zx_ioctl_t  func = zx_ioctls[nr];

    long ret;

    ret = (long)func(filp->private_data, cmd, arg);

    return ret;
}


static int zx_mmap(struct file *filp, struct vm_area_struct *vma)
{
    int ret = 0;
#ifndef __frv__
    zx_file_t     *priv = filp->private_data;
    zx_card_t     *card = priv->card;
    zx_map_argu_t *map  = priv->map;
    zx_map_argu_t map_argu;

    if(map == NULL)
    {
        bus_config_t bus_config = {0};

        unsigned long offset = vma->vm_pgoff << PAGE_SHIFT;

        zx_get_bus_config(card->pdev, &bus_config);

        map_argu.flags.mem_type   = ZX_SYSTEM_IO;

        if((offset >= bus_config.mem_start_addr[0]) && 
           (offset <  bus_config.mem_end_addr[0]))
        {
            /* map fb as wc */
            map_argu.flags.cache_type = ZX_MEM_WRITE_COMBINED;
        }
        else
        {
            /* map mmio reg as uc*/
            map_argu.flags.cache_type = ZX_MEM_UNCACHED;
        }

        map = &map_argu;
    }

    switch(map->flags.mem_type)
    {
        case ZX_SYSTEM_IO:
            ret = zx_map_system_io(vma, map);
            break;
            
        case ZX_SYSTEM_RAM:
            ret = zx_map_system_ram(vma, map);
            break;

        default:
            zx_error("%s, unknown memory map type", __func__);
            ret = -1;
            break;
    }
#endif

    return ret;
}

static unsigned int zx_poll(struct file *filp, struct poll_table_struct *wait)
{
    zx_file_t *priv = filp->private_data;

    unsigned int mask = 0;

    poll_wait(filp, &priv->wait_queue, wait);

    if(priv->wait_events & priv->curr_events)
    {
        mask = POLLIN;
    }

    return mask;
}

static struct file_operations zx_fops = 
{
    .owner   = THIS_MODULE,
    .open    = zx_open,
    .release = zx_release,
    .poll    = zx_poll,
    .unlocked_ioctl   = zx_unlocked_ioctl,
#if defined(__x86_64__) && defined(HAVE_COMPAT_IOCTL)
    .compat_ioctl = zx_unlocked_ioctl,
#endif
    .mmap = zx_mmap,
};
#endif

struct class *zx_class = NULL;

static int __init zx_init(void)
{
    int ret = -ENOMEM;

    zx_class = class_create(THIS_MODULE, ZX_DEV_NAME);

    if(zx_class == NULL)
    {
        zx_error("class_create() failed in zx_init.\n");
    }

    zx_info("Shanghai Zhaoxin Semiconductor Co., Ltd.\n");
    //zx_info("Version: %0d.%02d.%02d%s Build on: %s\n", DRIVER_MAJOR, DRIVER_MINOR, DRIVER_PATCHLEVEL, DRIVER_BRANCH, DRIVER_DATE);    
    zx_info("Version: %0x.%02x.%02x Build on: %s\n", DRIVER_MAJOR, DRIVER_MINOR, DRIVER_PATCHLEVEL, DRIVER_DATE);    

#if ZX_MALLOC_TRACK | ZX_ALLOC_PAGE_TRACK | ZX_MAP_PAGES_TRACK | ZX_MAP_IO_TRACK

    zx_mem_track_init();

#endif
    ret = zx_register_driver();

    if(ret)
    {
        zx_error("register_driver() failed in zx_init. ret:%x.\n", ret);
    }

    return 0;
}

static void __exit zx_exit(void)
{
    zx_unregister_driver();

    if(zx_class != NULL)
    {
        class_destroy(zx_class);

        zx_class = NULL;
    }

    unregister_chrdev(ZX_MAJOR, "zx");

#if ZX_MALLOC_TRACK | ZX_ALLOC_PAGE_TRACK | ZX_MAP_PAGES_TRACK | ZX_MAP_IO_TRACK
    zx_mem_track_list_result();
#endif

    zx_info("exit driver.\n");
}

module_init(zx_init);
module_exit(zx_exit);
MODULE_LICENSE("GPL");

#ifndef KERNEL_2_4
module_param(zx_fb, int, 0);
module_param(zx_fb_mode, charp, 0);
module_param(zx_flip, int, 0);
#else
MODULE_PARM(zx_fb_mode, "s");
#endif

MODULE_PARM_DESC(zx_fb, "enable zx fb driver");
MODULE_PARM_DESC(zx_fb_mode, "Initial video mode:<xres>x<yres>-<depth>@<refresh>");
MODULE_PARM_DESC(zx_flip, "enable frame buffer flip support");

