/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __ZX_DEUBGFS_H__
#define __ZX_DEUBGFS_H__

#include "zx.h"
#include "zx_driver.h"

#define DEBUGFS_ESCAPE_CREATE   1
#define DEBUGFS_ZX_CREATE      2

#define DEBUGFS_HEAP_NUM        7
#define DEBUGFS_MEMTRACK        (DEBUGFS_HEAP_NUM + 1)

struct zx_debugfs_device;

typedef struct zx_debugfs_node
{
    int                         type;
    void                        *adapter;
    struct zx_debugfs_device   *dev;
    int                         id;
    char                        name[20];
    struct dentry               *node_dentry;
    int                         create_hint;
    int                         min_id;
    struct list_head            list_item;
    unsigned int                hDevice;

}zx_debugfs_node_t;

typedef struct zx_debugfs_device
{
    zx_card_t                      *zx;
    struct list_head                node_list;
    struct zx_debugfs_node         heap[DEBUGFS_HEAP_NUM];
    struct zx_debugfs_node         info;
    struct zx_debugfs_node         memtrack;
    struct dentry                   *debug_root;
    struct os_mutex                 *lock;

}zx_debugfs_device_t;

zx_debugfs_device_t* zx_debugfs_create_device(zx_card_t *zx);
int zx_debugfs_destroy_device(zx_debugfs_device_t* dev);
int zx_debugfs_add_node(zx_debugfs_device_t* dev, int type, int id, unsigned int handle, int create_hint);
int zx_debugfs_remove_node(zx_debugfs_device_t* dev, unsigned int handle);

#endif
