/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __ZX_DRIVER_H
#define __ZX_DRIVER_H

#include "zx.h"
#include "zx_def.h"
#include "zx_types.h"
#include "os_interface.h"
#include "kernel_interface.h"

#define ZX_MAX_CARDS         16


typedef struct zx_file zx_file_t;

typedef int (*zx_ioctl_t)(zx_file_t* priv, unsigned int cmd, unsigned long arg);

typedef int (*irq_func_t)(void*);

typedef struct
{
    unsigned int               size_in;
    unsigned int               size_out;
    int                        send_data[MAX_IPC_OPERATION_DATA];
    int                        reply_data[MAX_IPC_OPERATION_DATA];
}zx_ipc_data_t;

typedef struct
{
    void              *adapter;
#ifdef ZX_PCIE_BUS
    struct pci_dev    *pdev;
#else
    struct platform_device *pdev;
#endif
    struct device     *device; //zx device
    char              busId[64];
    int               len;
    int               index;

    struct os_mutex   *lock;

    int               msi_enable;

    irq_func_t        isr_func;
    void              *isr_bf_func;

    struct tasklet_struct tasklet;

    wait_queue_head_t wait_queue;

    spinlock_t        wait_list_lock;
    struct list_head  wait_list;

    unsigned long long v_sync_count[8];
    unsigned long long v_sync_timestamp[8];

    zx_iga_virtual_info_t iga_virtual_info[8];

    int               enter_vt;    /* if enter vt means leave console mode */
    unsigned int      zxfb_pending_events;
    int               zxfb_enable;
    int               vga16fb_enable;
    int               vesafb_enable;
    int               reserved_vmem;
    zx_mode_info_t   vesafb_mode;
    int               freezable_patch;

    struct fb_info    *info;
    struct led_classdev lcd;

    zx_mode_info_t       tv_mode;
    zx_output_status_t   output_status;

    zx_ipc_data_t        ipc_data[ZX_IPC_TYPE_COUNT];

    struct zx_sync_dev  *sync_dev;

    struct input_dev      *rc_dev;
    zx_keymap_t          *keyboard_rc;
    int                keyboard_rc_keynum;
    void                *debugfs_dev;
    struct device       *pci_device;
    struct backlight_device *backlight_dev;
}zx_card_t;

struct zx_file
{
    struct drm_file  *drm_file;
    zx_card_t        *card;

    void              *map;
    unsigned int      gpu_device;

    unsigned int      need_cleanup_drawable;
    unsigned int      server_index;

    int               hold_lock;
    int               freezable;
    struct os_mutex   *lock;

    unsigned int      curr_events;
    unsigned int      wait_events;
    struct list_head  wait_list;
    wait_queue_head_t wait_queue;
};

typedef struct _cbiosProcReadInfo
{
	unsigned int DeviceId;
	unsigned int vbiosVersion;
	unsigned int Eclk;
	unsigned int Iclk;
	unsigned int Vclk;
	unsigned int VcpClk;
	//driver version
	unsigned int DrvVersion;
	unsigned char ReleaseDate[24];
}cbiosProcReadInfo;


extern char *zx_fb_mode;
extern int   zx_fb;
extern int   zx_flip;

extern struct class *zx_class;
extern zx_card_t    zx_cards[ZX_MAX_CARDS];
extern zx_ioctl_t   zx_ioctls[];
extern zx_ioctl_t   zx_ioctls_compat[];

extern int  zx_card_init(zx_card_t *zx, void *pdev);
extern int  zx_card_deinit(zx_card_t *zx);
extern void zx_card_pre_init(zx_card_t *zx, void *pdev);

extern void zx_interrupt_init(zx_card_t *zx);
extern void zx_interrupt_reinit(zx_card_t *zx);
extern void zx_interrupt_deinit(zx_card_t *zx);

extern int  zx_map_system_io(struct vm_area_struct *vma, zx_map_argu_t *map);
extern int  zx_map_system_ram(struct vm_area_struct *vma, zx_map_argu_t *map);

extern int  zx_add_wait(zx_card_t* zx, zx_file_t *priv, unsigned int);
extern int  zx_remove_wait(zx_card_t* zx, zx_file_t* priv);
extern void zx_get_event(zx_card_t* zx, zx_file_t* priv, zx_wait_func_t *wait);
extern int  zx_post_event(zx_card_t *zx, unsigned int events);
extern int  zx_ipc_operation(zx_card_t *zx, zx_file_t *priv, zx_ipc_operation_t *ipc_operation);

extern void  zx_init_2d_screen(zx_card_t *card, zx_file_t *priv, zx_2d_screen_t *screen);
extern void  zx_deinit_2d_screen(zx_card_t *card, zx_2d_screen_t *screen);
extern void  zx_cleanup_2d_screen(zx_card_t *card, zx_file_t *priv);

extern int   zx_fb_probe(zx_card_t *zx, char *mode_string, int flip);
extern int   zx_fb_shutdown(zx_card_t *zx);
extern int   zx_fb_remove(zx_card_t *zx);
extern void  zx_fb_set_pm(zx_card_t *zx, int bsuspend);
extern void  zx_fb_detect_vesa(zx_card_t *zx);

#ifdef NEW_ZXFB
extern int   zxfb_probe(zx_card_t *zx, char *mode_string, int flip);
extern int   zxfb_shutdown(zx_card_t *zx);
extern int   zxfb_remove(zx_card_t *zx);
extern void  zxfb_set_pm(zx_card_t *zx, int bsuspend);
extern void  zxfb_detect_vesa(zx_card_t *zx);
extern int   zxfb_handle_event(zx_card_t *zx, unsigned int events);

#define   zx_fb_probe          zxfb_probe
#define   zx_fb_shutdown       zxfb_shutdown
#define   zx_fb_remove         zxfb_remove
#define   zx_fb_set_pm         zxfb_set_pm
#define   zx_fb_detect_vesa    zxfb_detect_vesa

#else

extern int   zx_fb_probe(zx_card_t *zx, char *mode_string, int flip);
extern int   zx_fb_shutdown(zx_card_t *zx);
extern int   zx_fb_remove(zx_card_t *zx);
extern void  zx_fb_set_pm(zx_card_t *zx, int bsuspend);
extern void  zx_fb_detect_vesa(zx_card_t *zx);

#endif

extern void zx_init_bus_id(zx_card_t *zx);
extern int  zx_register_driver(void);
extern void zx_unregister_driver(void);
extern int  zx_register_interrupt(zx_card_t *zx, void *isr);
extern void zx_unregister_interrupt(zx_card_t *zx);

extern int zx_bo_fd_to_handle(zx_file_t *priv, zx_handle_fd_t *hfd);
extern int zx_bo_handle_to_fd(zx_file_t *priv, zx_handle_fd_t *hfd);

extern int  zx_create_allocation(int card, zx_create_allocation_t *create_data);
extern void zx_destroy_allocation(int card, zx_destroy_allocation_t *destroy_data);
extern int  zx_lock_allocation(int card, zx_lock_allocation_t *lock_data);
extern void zx_unlock_allocation(int card, zx_unlock_allocation_t *unlock_data);
extern void zx_flush_allocation(int card, zx_flush_allocation_t *flush_data);
extern void zx_open_allocation(int card, zx_open_allocation_t *open_data);
extern int  zx_create_device(int card, unsigned int *device);
extern void zx_destroy_device(int card, unsigned int device);

extern int  zx_sync_init(zx_card_t *zx);
extern void zx_sync_deinit(zx_card_t *zx);
extern void zx_keyboard_init(zx_card_t *zx);

extern void zx_begin_end_isp_dvfs(int card, unsigned int o);


extern int zx_ioctl_addctx(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_rmctx(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_lock_ctx(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_unlock_ctx(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_busid(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_wait_func(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_display_info(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_iga_info(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_remove_drawable(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_drawsize(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_device_comb(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_hpd_info(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_query_display_caps(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_set_mode(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_register_mhl_callback(zx_file_t* priv, unsigned int cmd, unsigned long arg);extern int zx_ioctl_get_hdmi_audio_format(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_cursor(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_dpms(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_mode_timing(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_2d_init(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_wait_chip_idle(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_wait_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_acquire_aperture(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_release_aperture(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_leave_vt(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_enter_vt(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_check_device_combination(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_set_ps(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_flip(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_enum_sub_device(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_create_allocation(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_destroy_allocation(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_unlock_allocation(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_flush_allocation(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_set_interactive(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_open_allocation(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_wait_allocation_idle(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_video_get_hw_caps(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_query_info(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_create_stream(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_destroy_stream(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_display_frame(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_cec_ctl(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_present(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_drawable_surface(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_vsync_info(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_create_device(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_destroy_device(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_begin_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_begin_miu_dump_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_end_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_end_miu_dump_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_send_perf_event(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_perf_status(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_destroy_context(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_create_di_context(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_destroy_di_context(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_query_chip_id(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_hdcp_op(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_create_fence_sync_object(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_destroy_fence_sync_object(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_wait_fence_sync_object(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_fence_value(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_create_fence_fd(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_flip_limit(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_wait_context_idle(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_mhl_cbus_send_command(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_set_power_perf_mode(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_gfx_stm(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_fd_to_handle(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_handle_to_fd(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_ust_msc(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_get_allocation_state(zx_file_t* priv, unsigned int cmd, unsigned long arg);
extern int zx_ioctl_iga_virtual_info(zx_file_t *priv, unsigned int cmd, unsigned long arg);
extern void zx_cleanup_drawable(zx_file_t *priv);
extern int zx_ioctl_access_aux(zx_file_t *priv, unsigned int cmd, unsigned long arg);

#endif

