/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __ZX_IOCTL_H__
#define __ZX_IOCTL_H__

#if defined(__linux__)
#include <asm/ioctl.h>
#define ZX_IOCTL_NR(n)         _IOC_NR(n)
#define ZX_IOCTL_TYPE(n)       _IOC_TYPE(n)
#define ZX_IOC_VOID            _IOC_NONE
#define ZX_IOC_READ            _IOC_READ
#define ZX_IOC_WRITE           _IOC_WRITE
#define ZX_IOC_READWRITE       _IOC_READ | _IOC_WRITE
#define ZX_IOC(dir, group, nr, size) _IOC(dir, group, nr, size)
#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
#if defined(__FreeBSD__) && defined(IN_MODULE)
#undef ioctl
#include <sys/ioccom.h>
#define ioctl(a,b,c)            xf86ioctl(a,b,c)
#else
#include <sys/ioccom.h>
#endif
#define ZX_IOCTL_NR(n)         ((n) & 0xff)
#define ZX_IOCTL_TYPE(n)       (((n) >> 8) & 0xff)
#define ZX_IOC_VOID            IOC_VOID
#define ZX_IOC_READ            IOC_OUT
#define ZX_IOC_WRITE           IOC_IN
#define ZX_IOC_READWRITE       IOC_INOUT
#define ZX_IOC(dir, group, nr, size) _IOC(dir, group, nr, size)
#endif

#ifndef __user
#define __user
#endif

enum zx_ioctl_nr
{
    ioctl_nr_getbusid,
    ioctl_nr_get_displayinfo,
    ioctl_nr_get_igainfo,  
    ioctl_nr_addctx,
    ioctl_nr_rmctx,
    ioctl_nr_lock,
    ioctl_nr_unlock,
    ioctl_nr_clipnotify,
    ioctl_nr_adddraw,
    ioctl_nr_rmdraw,
    ioctl_nr_getdrawsize,
    ioctl_nr_query_display_caps,
    ioctl_nr_query_output_caps,
    ioctl_nr_mode_set,
    ioctl_nr_cursor,
    ioctl_nr_gamma,
    ioctl_nr_dpms,
    ioctl_nr_get_mode_timing,
    ioctl_nr_leave_vt,
    ioctl_nr_enter_vt,
    ioctl_nr_check_device_combination,
    ioctl_nr_2d_init,
    ioctl_nr_acquire_aperture,
    ioctl_nr_release_aperture,
    ioctl_nr_wait_chip_idle,
    ioctl_nr_escape_call,
    ioctl_nr_2d_draw,
    ioctl_nr_init_screen,
    ioctl_nr_deinit_screen,
    ioctl_nr_set_ps,
    ioctl_nr_enum_sub_device,
    ioctl_nr_create_allocation,
    ioctl_nr_destroy_allocation,
    ioctl_nr_lock_allocation,
    ioctl_nr_unlock_allocation,
    ioctl_nr_open_allocation,
    ioctl_nr_wait_allocation_idle,
    ioctl_nr_video_get_hw_caps,
    ioctl_nr_get_address_config,
    ioctl_nr_query_info,
    ioctl_nr_wait_func,
    ioctl_nr_flip,
    ioctl_nr_cec_ctl,
    ioctl_nr_present,
    ioctl_nr_get_drawable_surface,
    ioctl_nr_get_vsync_info,
    ioctl_nr_create_device,
    ioctl_nr_destroy_device,
    ioctl_nr_begin_perf_event,
    ioctl_nr_end_perf_event,
    ioctl_nr_get_perf_event,
    ioctl_nr_send_perf_event,
    ioctl_nr_get_perf_status,
    ioctl_nr_begin_miu_dump_perf_event,
    ioctl_nr_end_miu_dump_perf_event,
    ioctl_nr_set_miu_reg_list_perf_event,
    ioctl_nr_get_miu_dump_perf_event,
    ioctl_nr_direct_get_miu_dump_perf_event,
    ioctl_nr_create_context,
    ioctl_nr_destroy_context,
    ioctl_nr_wait_context_idle,
    ioctl_nr_render,
    ioctl_nr_flush_allocation,
    ioctl_nr_query_chip_id,
    ioctl_nr_hdcp_op,
    ioctl_nr_create_fence_sync_object,
    ioctl_nr_destroy_fence_sync_object,
    ioctl_nr_wait_fence_sync_object,
    ioctl_nr_fence_value,
    ioctl_nr_create_fence_fd, /* it destroy by close(fd)*/
    ioctl_nr_flip_limit,
    ioctl_nr_ipc_operation,
    ioctl_nr_create_stream,
    ioctl_nr_destroy_stream,
    ioctl_nr_display_frame,
    ioctl_nr_set_interactive,
    ioctl_nr_send_dsi_cmd,
    ioctl_nr_mhl_cbus_send_cmd,
    ioctl_nr_create_di_context,
    ioctl_nr_destroy_di_context,
    ioctl_nr_register_mhl_callback,
    ioctl_nr_set_power_perf_mode,
    ioctl_nr_get_device_comb,
    ioctl_nr_get_hpd_info,
    ioctl_nr_gfx_stm,
    ioctl_nr_get_hdmi_audio_format,
    ioctl_nr_wait_event,
    ioctl_nr_handle_to_fd,
    ioctl_nr_fd_to_handle,
    ioctl_nr_get_ust_msc,
    ioctl_nr_get_allocation_state,
    ioctl_nr_video_damage,
    ioctl_nr_iga_virtual_info,
    ioctl_nr_access_aux,
};

/* zx_fence sync ioctls */
enum zx_sync_ioctl_nr
{
    ioctl_nr_sync_wait,
    ioctl_nr_sync_merge,
    ioctl_nr_sync_info,
};

enum zx_buffer_object_ioctl_nr
{
    ioctl_nr_get_read_fence,
    ioctl_nr_get_write_fence,
    ioctl_nr_set_read_fence,
    ioctl_nr_set_write_fence,
    ioctl_nr_get_buffer_data,
    ioctl_nr_set_buffer_data,
};

#define ZX_DIR_NAME "/dev/dri"
#define ZX_DEV_NAME "card"
#define ZX_PROC_NAME "driver/dri"
#define CBIOS_PROC_NAME "driver/cbios"
#define ZX_AUX_PROC_NAME "driver/aux"
#define DEV_MEM      "/dev/mem"

#define ZX_IOCTL_BASE                     's'
#define ZX_IO(nr)                         _IO(ZX_IOCTL_BASE, nr)
#define ZX_IOR(nr, type)                  _IOR(ZX_IOCTL_BASE, nr, type)
#define ZX_IOW(nr, type)                  _IOW(ZX_IOCTL_BASE, nr, type)
#define ZX_IOWR(nr, type)                 _IOWR(ZX_IOCTL_BASE, nr, type)

#define ZX_SYNC_IOCTL_BASE                's'
#define ZX_SYNC_IO(nr)                     _IO(ZX_SYNC_IOCTL_BASE, nr)
#define ZX_SYNC_IOR(nr, type)              _IOR(ZX_SYNC_IOCTL_BASE, nr, type)
#define ZX_SYNC_IOW(nr, type)              _IOW(ZX_SYNC_IOCTL_BASE, nr, type)
#define ZX_SYNC_IOWR(nr, type)             _IOWR(ZX_SYNC_IOCTL_BASE, nr, type)

#define ZX_IOCTL_GET_BUSID                ZX_IOWR(ioctl_nr_getbusid, zx_unique_t)
#define ZX_IOCTL_GET_DISPLAYINFO          ZX_IOWR(ioctl_nr_get_displayinfo, zx_display_info_t)
#define ZX_IOCTL_GET_IGAINFO              ZX_IOWR(ioctl_nr_get_igainfo, zx_iga_info_t)
#define ZX_IOCTL_ADD_CTX                  ZX_IOWR(ioctl_nr_addctx, zx_ctx_t)
#define ZX_IOCTL_RM_CTX                   ZX_IOWR(ioctl_nr_rmctx, zx_ctx_t)
#define ZX_IOCTL_LOCK                     ZX_IOWR(ioctl_nr_lock, zx_lock_t)
#define ZX_IOCTL_UNLOCK                   ZX_IOWR(ioctl_nr_unlock, zx_lock_t)
#define ZX_IOCTL_CLIP_NOTIFY              ZX_IOWR(ioctl_nr_clipnotify, zx_clipnotify_t)
#define ZX_IOCTL_ADD_DRAW                 ZX_IOWR(ioctl_nr_adddraw, zx_drawable_t)
#define ZX_IOCTL_RM_DRAW                  ZX_IOWR(ioctl_nr_rmdraw, int)
#define ZX_IOCTL_GET_DRAWSIZE             ZX_IOWR(ioctl_nr_getdrawsize, zx_get_drawsize_t)
#define ZX_IOCTL_QUERY_DISPLAY_CAPS       ZX_IOWR(ioctl_nr_query_display_caps, zx_display_caps_t)
#define ZX_IOCTL_QUERY_OUTPUT_CAPS        ZX_IOWR(ioctl_nr_query_output_caps, zx_output_caps_t)
#define ZX_IOCTL_MODE_SET                 ZX_IOWR(ioctl_nr_mode_set, zx_mode_info_t)
#define ZX_IOCTL_CURSOR                   ZX_IOWR(ioctl_nr_cursor, zx_set_cursor_t)
#define ZX_IOCTL_GAMMA                    ZX_IOWR(ioctl_nr_gamma, zx_gamma_info_t)
#define ZX_IOCTL_DPMS                     ZX_IOWR(ioctl_nr_dpms, zx_dpms_set_t)
#define ZX_IOCTL_GET_MODE_TIMING          ZX_IOWR(ioctl_nr_get_mode_timing, zx_mode_timing_info_t)
#define ZX_IOCTL_LEAVE_VT                 ZX_IO(ioctl_nr_leave_vt)
#define ZX_IOCTL_ENTER_VT                 ZX_IO(ioctl_nr_enter_vt)
#define ZX_IOCTL_CHECK_DEVICE_COMBINATION ZX_IOWR(ioctl_nr_check_device_combination, zx_check_device_combination_t)
#define ZX_IOCTL_2D_INIT                  ZX_IOW(ioctl_nr_2d_init, zx_2d_init_t)
#define ZX_IOCTL_ACQUIRE_APERTURE         ZX_IOW(ioctl_nr_acquire_aperture, zx_acquire_aperture_t)
#define ZX_IOCTL_RELEASE_APERTURE         ZX_IOW(ioctl_nr_release_aperture, zx_release_aperture_t)
#define ZX_IOCTL_WAIT_CHIP_IDLE           ZX_IO(ioctl_nr_wait_chip_idle)
#define ZX_IOCTL_ESCAPE_CALL              ZX_IOW(ioctl_nr_escape_call, long)
#define ZX_IOCTL_2D_DRAW                  ZX_IOW(ioctl_nr_2d_draw, long)
#define ZX_IOCTL_INIT_SCREEN              ZX_IOW(ioctl_nr_init_screen, zx_2d_screen_t)
#define ZX_IOCTL_DEINIT_SCREEN            ZX_IOW(ioctl_nr_deinit_screen, zx_2d_screen_t)
#define ZX_IOCTL_SET_PS                   ZX_IOW(ioctl_nr_set_ps, zx_ps_info_t)
#define ZX_IOCTL_ENUM_SUB_DEVICE          ZX_IOW(ioctl_nr_enum_sub_device, void*)
#define ZX_IOCTL_CREATE_ALLOCATION        ZX_IOWR(ioctl_nr_create_allocation, zx_create_allocation_t)
#define ZX_IOCTL_DESTROY_ALLOCATION       ZX_IOW(ioctl_nr_destroy_allocation, unsigned int)
#define ZX_IOCTL_IPC_OPERATION            ZX_IOWR(ioctl_nr_ipc_operation, zx_ipc_operation_t)
#define ZX_IOCTL_SET_INTERACTIVE          ZX_IOWR(ioctl_nr_set_interactive, int)
#define ZX_IOCTL_LOCK_ALLOCATION          ZX_IOWR(ioctl_nr_lock_allocation, zx_lock_allocation_t)
#define ZX_IOCTL_UNLOCK_ALLOCATION        ZX_IOW(ioctl_nr_unlock_allocation, zx_unlock_allocation_t)
#define ZX_IOCTL_FLUSH_ALLOCATION         ZX_IOW(ioctl_nr_flush_allocation, zx_flush_allocation_t)
#define ZX_IOCTL_OPEN_ALLOCATION          ZX_IOWR(ioctl_nr_open_allocation, zx_open_allocation_t)
#define ZX_IOCTL_WAIT_ALLOCATION_IDLE     ZX_IOW(ioctl_nr_wait_allocation_idle, zx_wait_allocation_idle_t)
#define ZX_IOCTL_VIDEO_GET_HW_CAPS        ZX_IOWR(ioctl_nr_video_get_hw_caps, unsigned char)
#define ZX_IOCTL_GET_ADDRESS_CONFIG       ZX_IOWR(ioctl_nr_get_address_config, zx_get_address_config_t)
#define ZX_IOCTL_QUERY_INFO               ZX_IOWR(ioctl_nr_query_info, zx_query_info_t)
#define ZX_IOCTL_WAIT_FUNC                ZX_IOWR(ioctl_nr_wait_func, zx_wait_func_t)
#define ZX_IOCTL_FLIP                     ZX_IOW(ioctl_nr_flip, zx_flip_info_t)
#define ZX_IOCTL_CREATE_STREAM            ZX_IOWR(ioctl_nr_create_stream, zx_create_stream_t)
#define ZX_IOCTL_DESTROY_STREAM           ZX_IOW(ioctl_nr_destroy_stream, zx_destroy_stream_t)
#define ZX_IOCTL_DISPLAY_FRAME            ZX_IOWR(ioctl_nr_display_frame, zx_display_frame_t)
#define ZX_IOCTL_CEC_CTL                  ZX_IOWR(ioctl_nr_cec_ctl, zx_cec_ctl_t)
#define ZX_IOCTL_PRESENT                  ZX_IOWR(ioctl_nr_present, zx_present_t)
#define ZX_IOCTL_GET_DRAWABLE_SURFACE     ZX_IOWR(ioctl_nr_get_drawable_surface, zx_get_drawable_surface_t)
#define ZX_IOCTL_GET_VSYNC_INFO           ZX_IOWR(ioctl_nr_get_vsync_info, zx_get_vsync_info_t)
#define ZX_IOCTL_CREATE_DEVICE            ZX_IOWR(ioctl_nr_create_device, zx_create_device_t)
#define ZX_IOCTL_DESTROY_DEVICE           ZX_IOWR(ioctl_nr_destroy_device, unsigned int)
#define ZX_IOCTL_BEGIN_PERF_EVENT         ZX_IOW(ioctl_nr_begin_perf_event, zx_begin_perf_event_t)
#define ZX_IOCTL_END_PERF_EVENT           ZX_IOW(ioctl_nr_end_perf_event, zx_end_perf_event_t)
#define ZX_IOCTL_GET_PERF_EVENT           ZX_IOWR(ioctl_nr_get_perf_event, zx_get_perf_event_t)
#define ZX_IOCTL_SEND_PERF_EVENT          ZX_IOW(ioctl_nr_send_perf_event, zx_perf_event_t)
#define ZX_IOCTL_GET_PERF_STATUS          ZX_IOWR(ioctl_nr_get_perf_status, zx_perf_status_t)
#define ZX_IOCTL_BEGIN_MIU_DUMP_PERF_EVENT ZX_IOWR(ioctl_nr_begin_miu_dump_perf_event, zx_begin_miu_dump_perf_event_t)
#define ZX_IOCTL_END_MIU_DUMP_PERF_EVENT  ZX_IOWR(ioctl_nr_end_miu_dump_perf_event, zx_end_miu_dump_perf_event_t)
#define ZX_IOCTL_SET_MIU_REG_LIST_PERF_EVENT ZX_IOWR(ioctl_nr_set_miu_reg_list_perf_event, zx_miu_reg_list_perf_event_t)
#define ZX_IOCTL_GET_MIU_DUMP_PERF_EVENT  ZX_IOWR(ioctl_nr_get_miu_dump_perf_event, zx_get_miu_dump_perf_event_t)
#define ZX_IOCTL_DIRECT_GET_MIU_DUMP_PERF_EVENT ZX_IOWR(ioctl_nr_direct_get_miu_dump_perf_event, zx_direct_get_miu_dump_perf_event_t)
#define ZX_IOCTL_CREATE_CONTEXT           ZX_IOWR(ioctl_nr_create_context, zx_create_context_t)
#define ZX_IOCTL_DESTROY_CONTEXT          ZX_IOWR(ioctl_nr_destroy_context, zx_destroy_context_t)
#define ZX_IOCTL_RENDER                   ZX_IOWR(ioctl_nr_render, zx_render_t)
#define ZX_IOCTL_QUERY_CHIP_ID            ZX_IOWR(ioctl_nr_query_chip_id, zx_chip_id_t)
#define ZX_IOCTL_HDCP_OP                  ZX_IOWR(ioctl_nr_hdcp_op, zx_hdcp_op_t)
#define ZX_IOCTL_CREATE_FENCE_SYNC_OBJECT ZX_IOWR(ioctl_nr_create_fence_sync_object, zx_create_fence_sync_object_t)
#define ZX_IOCTL_DESTROY_FENCE_SYNC_OBJECT ZX_IOWR(ioctl_nr_destroy_fence_sync_object, zx_destroy_fence_sync_object_t)
#define ZX_IOCTL_WAIT_FENCE_SYNC_OBJECT   ZX_IOWR(ioctl_nr_wait_fence_sync_object, zx_wait_fence_sync_object_t)
#define ZX_IOCTL_FENCE_VALUE              ZX_IOWR(ioctl_nr_fence_value, zx_fence_value_t)
#define ZX_IOCTL_SEND_DSI_CMD             ZX_IOWR(ioctl_nr_send_dsi_cmd, zx_dsi_cmd_t)
#define ZX_IOCTL_MHL_CBUS_SEND_CMD        ZX_IOWR(ioctl_nr_mhl_cbus_send_cmd, zx_cbus_cmd_t)
#define ZX_IOCTL_CREATE_DI_CONTEXT        ZX_IOWR(ioctl_nr_create_di_context, zx_create_di_context_t)
#define ZX_IOCTL_DESTROY_DI_CONTEXT       ZX_IOWR(ioctl_nr_destroy_di_context, zx_destroy_di_context_t)
#define ZX_IOCTL_CREATE_CONTEXT           ZX_IOWR(ioctl_nr_create_context, zx_create_context_t)
#define ZX_IOCTL_DESTROY_CONTEXT          ZX_IOWR(ioctl_nr_destroy_context, zx_destroy_context_t)
#define ZX_IOCTL_RENDER                   ZX_IOWR(ioctl_nr_render, zx_render_t)
#define ZX_IOCTL_GET_DEVICE_COMB          ZX_IOWR(ioctl_nr_get_device_comb, zx_dev_comb_t)
#define ZX_IOCTL_GET_HPD_INFO             ZX_IOWR(ioctl_nr_get_hpd_info, zx_hpd_info_t)

#define ZX_IOCTL_CREATE_FENCE_FD          ZX_IOWR(ioctl_nr_create_fence_fd,zx_create_fence_fd_t)
#define ZX_IOCTL_REGISTER_MHL_CALLBACK    ZX_IO(ioctl_nr_register_mhl_callback)


/*zx fence sync ioctls*/
#define ZX_SYNC_IOCTL_WAIT     ZX_SYNC_IOWR(ioctl_nr_sync_wait,zx_wait_fence_t)
#define ZX_SYNC_IOCTL_INFO     ZX_SYNC_IOWR(ioctl_nr_sync_info,zx_fence_info_t)
#define ZX_SYNC_IOCTL_MERGE    ZX_SYNC_IOWR(ioctl_nr_sync_merge,zx_fence_merge_t)

#define ZX_IOCTL_SET_FLIP_COUNTER         ZX_IOWR(ioctl_nr_flip_limit,zx_flip_limit_t)

#define ZX_IOCTL_WAIT_CONTEXT_IDLE        ZX_IOWR(ioctl_nr_wait_context_idle, unsigned int)

#define ZX_IOCTL_SET_READ_FENCE           ZX_IOR(ioctl_nr_set_read_fence, zx_bo_fence_t)
#define ZX_IOCTL_SET_WRITE_FENCE          ZX_IOR(ioctl_nr_set_write_fence, zx_bo_fence_t)
#define ZX_IOCTL_GET_READ_FENCE           ZX_IOWR(ioctl_nr_get_read_fence, zx_bo_fence_t)
#define ZX_IOCTL_GET_WRITE_FENCE          ZX_IOWR(ioctl_nr_get_write_fence, zx_bo_fence_t)

#define ZX_IOCTL_GET_BUFFER_DATA          ZX_IOWR(ioctl_nr_get_buffer_data, zx_bo_data_t)
#define ZX_IOCTL_SET_BUFFER_DATA          ZX_IOWR(ioctl_nr_set_buffer_data, zx_bo_data_t)
#define ZX_IOCTL_GET_HDMI_AUDIO_FORMAT    ZX_IOR(ioctl_nr_get_hdmi_audio_format, zx_hdmi_audio_formats)
#define ZX_IOCTL_SET_POWER_PERF_MODE      ZX_IOWR(ioctl_nr_set_power_perf_mode, zx_power_perf_mode_info_t)
#define ZX_IOCTL_GFX_STM                  ZX_IOWR(ioctl_nr_gfx_stm, unsigned int *)
#define ZX_IOCTL_WAIT_EVENT        ZX_IOW(ioctl_nr_wait_event, zx_wait_func_t)
#define ZX_IOCTL_HANDLE_TO_FD          ZX_IOW(ioctl_nr_handle_to_fd, zx_handle_fd_t)
#define ZX_IOCTL_FD_TO_HANDLE          ZX_IOW(ioctl_nr_fd_to_handle, zx_handle_fd_t)
#define ZX_IOCTL_GET_UST_MSC           ZX_IOWR(ioctl_nr_get_ust_msc, zx_ust_msc_t)
#define ZX_IOCTL_GET_ALLOCATION_STATE           ZX_IOWR(ioctl_nr_get_allocation_state, zx_get_allocation_state_t)
#define ZX_IOCTL_VIDEO_DAMAGE         ZX_IOW(ioctl_nr_video_damage, zx_video_damage_t)
#define ZX_IOCTL_IGA_VIRTUAL_INFO         ZX_IOW(ioctl_nr_iga_virtual_info, zx_iga_virtual_info_t)
#define ZX_IOCTL_ACCESS_AUX    ZX_IOWR(ioctl_nr_access_aux, zx_aux_t)

/* for fb Driver private ioctl */
/*zx + index */
#define FB_ZX_IOCTL_GET_ALLOCATION             0x73336701
#define FB_ZX_IOCTL_SET_CURSOR                 0x73336702
#define FB_ZX_IOCTL_QUEUE_POST_ALLOCATION      0x73336703
#define FB_ZX_IOCTL_DEQUEUE_POST_ALLOCATION    0x73336704
#define FB_ZX_IOCTL_GET_FRAMEBUFFER_INFO       0x73336705
#define FB_ZX_IOCTL_SET_ROTATION               0x73336706

#endif //__ZX_IOCTL_H__

