/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include"os_interface.h"
#include"zx_ioctl.h"
#include"zx_ioctl_compat.h"
#include"zx_def.h"
#include"zx_driver.h"
#include "kernel_interface.h"
#include "zx_screen.h"
#include "zx_sync.h"
#include "zx_buffer_object.h"
#include "zx_debugfs.h"

static unsigned long compat_ulong(compat_ulong_t uulong)
{
    return (unsigned long)uulong;
}

static compat_ulong_t ulong_to_compat(unsigned long uulong)
{
    return (compat_ulong_t)uulong;
}

static int zx_ioctl_clip_notify_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t                    *zx  = priv->card;
    void __user                   *argp = (void __user*)arg;
    zx_clipnotify_compat_t       clip_compat;
    zx_clipnotify_t              clip;

    if(zx_copy_from_user(&clip_compat, argp, sizeof(clip_compat)))
    {
        return -1;
    }

    clip.x = clip_compat.x;
    clip.y = clip_compat.y;
    clip.width = clip_compat.width;
    clip.height = clip_compat.height;
    clip.screen_x = clip_compat.screen_x;
    clip.screen_y = clip_compat.screen_y;
    clip.numClipRects = clip_compat.numClipRects;
    clip.xId = clip_compat.xId;
    clip.kdrawId = clip_compat.kdrawId;
    clip.scrnIndex = clip_compat.scrnIndex;
    clip.iga_mask = clip_compat.iga_mask;
    clip.hAllocation = clip_compat.hAllocation;
    clip.clipRects = compat_ptr(clip_compat.clipRects);
    clip.clipRectsVol = clip_compat.clipRectsVol;

    zx_clip_notify(zx, &clip);

    return 0;
}

static int zx_ioctl_add_drawable_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t                    *zx  = priv->card;
    void __user                   *argp = (void __user*)arg;
    zx_drawable_compat_t         drawable_compat;
    zx_drawable_t                drawable;
    int i = 0;

    if(zx_copy_from_user(&drawable_compat, argp, sizeof(zx_drawable_compat_t)))
    {
        return -1;
    }

    drawable.server_index = drawable_compat.server_index;
    drawable.xId          = drawable_compat.xId;
    for(i=0; i<MAX_SCREENS; i++)
    {
        drawable.clipNotify[i].x = drawable_compat.clipNotify[i].x;
        drawable.clipNotify[i].y = drawable_compat.clipNotify[i].y;
        drawable.clipNotify[i].width = drawable_compat.clipNotify[i].width;
        drawable.clipNotify[i].height = drawable_compat.clipNotify[i].height;
        drawable.clipNotify[i].screen_x = drawable_compat.clipNotify[i].screen_x;
        drawable.clipNotify[i].screen_y = drawable_compat.clipNotify[i].screen_y;
        drawable.clipNotify[i].numClipRects = drawable_compat.clipNotify[i].numClipRects;
        drawable.clipNotify[i].xId = drawable_compat.clipNotify[i].xId;
        drawable.clipNotify[i].kdrawId = drawable_compat.clipNotify[i].kdrawId;
        drawable.clipNotify[i].scrnIndex = drawable_compat.clipNotify[i].scrnIndex;
        drawable.clipNotify[i].iga_mask = drawable_compat.clipNotify[i].iga_mask;
        drawable.clipNotify[i].hAllocation = drawable_compat.clipNotify[i].hAllocation;
        drawable.clipNotify[i].clipRects = compat_ptr(drawable_compat.clipNotify[i].clipRects);
        drawable.clipNotify[i].clipRectsVol = drawable_compat.clipNotify[i].clipRectsVol;
    }
    drawable.refCount = drawable_compat.refCount;

    zx_add_drawable(zx, &drawable);

    drawable_compat.server_index = drawable.server_index;
    drawable_compat.xId          = drawable.xId;
    for(i=0; i<MAX_SCREENS; i++)
    {
        drawable_compat.clipNotify[i].x = drawable.clipNotify[i].x;
        drawable_compat.clipNotify[i].y = drawable.clipNotify[i].y;
        drawable_compat.clipNotify[i].width = drawable.clipNotify[i].width;
        drawable_compat.clipNotify[i].height = drawable.clipNotify[i].height;
        drawable_compat.clipNotify[i].screen_x = drawable.clipNotify[i].screen_x;
        drawable_compat.clipNotify[i].screen_y = drawable.clipNotify[i].screen_y;
        drawable_compat.clipNotify[i].numClipRects = drawable.clipNotify[i].numClipRects;
        drawable_compat.clipNotify[i].xId = drawable.clipNotify[i].xId;
        drawable_compat.clipNotify[i].kdrawId = drawable.clipNotify[i].kdrawId;
        drawable_compat.clipNotify[i].scrnIndex = drawable.clipNotify[i].scrnIndex;
        drawable_compat.clipNotify[i].iga_mask = drawable.clipNotify[i].iga_mask;
        drawable_compat.clipNotify[i].hAllocation = drawable.clipNotify[i].hAllocation;
        drawable_compat.clipNotify[i].clipRects = ptr_to_compat(drawable.clipNotify[i].clipRects);
        drawable_compat.clipNotify[i].clipRectsVol = drawable.clipNotify[i].clipRectsVol;
    }
    drawable_compat.refCount = drawable.refCount;

    if(zx_copy_to_user(argp, &drawable_compat, sizeof(zx_drawable_compat_t)))
    {
        return -1;
    }

    return 0;
}

static int zx_ioctl_query_output_caps_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t            *zx  = priv->card;
    void __user           *argp = (void __user*)arg;

    zx_output_caps_compat_t  output_caps_compat;
    zx_output_caps_t  output_caps;

    if(zx_copy_from_user(&output_caps_compat, argp, sizeof(zx_output_caps_compat_t)))
    {
        return -1;
    }

    output_caps.type = output_caps_compat.type;
    output_caps.monitor_type = output_caps_compat.monitor_type;
    output_caps.monitor_width = output_caps_compat.monitor_width;
    output_caps.monitor_height = output_caps_compat.monitor_height;
    output_caps.connected = output_caps_compat.connected;
    output_caps.edid_size = output_caps_compat.edid_size;
    output_caps.edid = compat_ptr(output_caps_compat.edid);
    output_caps.mode_num = output_caps_compat.mode_num;
    output_caps.modes = compat_ptr(output_caps_compat.modes);
    output_caps._3dmode_num = output_caps_compat._3dmode_num;
    output_caps._3dmodes = compat_ptr(output_caps_compat._3dmodes);
    output_caps.support_audio = output_caps_compat.support_audio;

    krnl_query_output_caps(zx->adapter, &output_caps, 1);

    output_caps_compat.type = output_caps.type;
    output_caps_compat.monitor_type = output_caps.monitor_type;
    output_caps_compat.monitor_width = output_caps.monitor_width;
    output_caps_compat.monitor_height = output_caps.monitor_height;
    output_caps_compat.connected = output_caps.connected;
    output_caps_compat.edid_size = output_caps.edid_size;
    output_caps_compat.edid = ptr_to_compat(output_caps.edid);
    output_caps_compat.mode_num = output_caps.mode_num;
    output_caps_compat.modes = ptr_to_compat(output_caps.modes);
    output_caps_compat._3dmode_num = output_caps._3dmode_num;
    output_caps_compat._3dmodes = ptr_to_compat(output_caps._3dmodes);
    output_caps_compat.support_audio = output_caps.support_audio;

    if(zx_copy_to_user(argp, &output_caps_compat, sizeof(output_caps_compat)))
    {
        return -1;
    }

    return 0;
}

static int zx_ioctl_gamma_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t            *zx  = priv->card;
    void __user           *argp = (void __user*)arg;

    int                   ret   = -1;

    zx_gamma_info_compat_t  gamma_compat = {0};
    zx_gamma_info_t  gamma = {0};

    unsigned int* lut_buffer   = zx_malloc(256 * sizeof(unsigned int));

    if(!zx_copy_from_user(&gamma_compat, argp, sizeof(gamma_compat)) 
       && !zx_copy_from_user(lut_buffer, compat_ptr(gamma_compat.pLutBuf), 256 * sizeof(unsigned int)))
    {
        gamma.crtc = gamma_compat.crtc;
        gamma.pLutBuf = lut_buffer;
    
        krnl_set_gamma(zx->adapter, &gamma);

        ret = 0;
    }

    zx_free(lut_buffer);

    return ret;
}

static int zx_ioctl_init_screen_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    void __user       *argp = (void __user*)arg;

    zx_2d_screen_compat_t   screen_compat;
    zx_2d_screen_t   screen;

    if(zx_copy_from_user(&screen_compat, argp, sizeof(zx_2d_screen_compat_t)))
    {
        return -1;
    }

    screen.server_index = screen_compat.server_index;
    screen.scrnIndex = screen_compat.scrnIndex;
    screen.master = screen_compat.master;
    screen.samm = screen_compat.samm;
    screen.mamm = screen_compat.mamm;
    screen.x = screen_compat.x;
    screen.y = screen_compat.y;
    screen.width = screen_compat.width;
    screen.height = screen_compat.height;
    screen.bpp = screen_compat.bpp;
    screen.pitch = screen_compat.pitch;
    screen.device = screen_compat.device;
    screen.hwFormat = screen_compat.hwFormat;
    screen.rotate = screen_compat.rotate;
    screen.gbd = screen_compat.gbd;
    screen.use_rxa = screen_compat.use_rxa;
    screen.bDuoView = screen_compat.bDuoView;
    screen.fbOffset = compat_ulong(screen_compat.fbOffset);
    screen.hAllocation = screen_compat.hAllocation;

    zx_init_2d_screen(priv->card, priv, &screen);

    screen_compat.server_index = screen.server_index;
    screen_compat.scrnIndex = screen.scrnIndex;
    screen_compat.master = screen.master;
    screen_compat.samm = screen.samm;
    screen_compat.mamm = screen.mamm;
    screen_compat.x = screen.x;
    screen_compat.y = screen.y;
    screen_compat.width = screen.width;
    screen_compat.height = screen.height;
    screen_compat.bpp = screen.bpp;
    screen_compat.pitch = screen.pitch;
    screen_compat.device = screen.device;
    screen_compat.hwFormat = screen.hwFormat;
    screen_compat.rotate = screen.rotate;
    screen_compat.gbd = screen.gbd;
    screen_compat.use_rxa = screen.use_rxa;
    screen_compat.bDuoView = screen.bDuoView;
    screen_compat.fbOffset = ulong_to_compat(screen.fbOffset);
    screen_compat.hAllocation = screen.hAllocation;

    if(zx_copy_to_user(argp, &screen, sizeof(zx_2d_screen_t)))
    {
        return -1;
    }

    return 0;
}

static int zx_ioctl_deinit_screen_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    void __user       *argp = (void __user*)arg;

    zx_2d_screen_compat_t   screen_compat;
    zx_2d_screen_t   screen;

    if(zx_copy_from_user(&screen_compat, argp, sizeof(zx_2d_screen_t)))
    {
        return -1;
    }

    screen.server_index = screen_compat.server_index;
    screen.scrnIndex = screen_compat.scrnIndex;
    screen.master = screen_compat.master;
    screen.samm = screen_compat.samm;
    screen.mamm = screen_compat.mamm;
    screen.x = screen_compat.x;
    screen.y = screen_compat.y;
    screen.width = screen_compat.width;
    screen.height = screen_compat.height;
    screen.bpp = screen_compat.bpp;
    screen.pitch = screen_compat.pitch;
    screen.device = screen_compat.device;
    screen.hwFormat = screen_compat.hwFormat;
    screen.rotate = screen_compat.rotate;
    screen.gbd = screen_compat.gbd;
    screen.use_rxa = screen_compat.use_rxa;
    screen.bDuoView = screen_compat.bDuoView;
    screen.fbOffset = compat_ulong(screen_compat.fbOffset);
    screen.hAllocation = screen_compat.hAllocation;

    zx_deinit_2d_screen(priv->card, &screen);

    return 0;
}

static int zx_ioctl_escape_call_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t                       *zx  = priv->card;
    void __user                      *argp = (void __user*)arg;

    krnl_hint_t                      hint  = {0};
    zx_escape_call_t                escape;
    int                              ret = 0;

    if(zx_copy_from_user(&escape, argp, sizeof(zx_escape_call_t)))
    {
        return -1;
    }

    hint.filp    = priv->drm_file->filp;
    hint.hDevice = priv->gpu_device;

    ret = krnl_escape_call_compat(zx->adapter, &hint, &escape);

    if(hint.create_device)
    {
        priv->gpu_device = hint.hDevice;
        if(zx->debugfs_dev)
        {
            zx_debugfs_add_node(zx->debugfs_dev, DEBUGFS_NODE_DEVICE, zx_get_current_pid(), priv->gpu_device, DEBUGFS_ESCAPE_CREATE);
        }
    }
    else if(hint.destroy_device)
    {
        if(zx->debugfs_dev)
        {
            zx_debugfs_remove_node(zx->debugfs_dev, priv->gpu_device);
        }

        priv->gpu_device = 0;
    }

    return ret;
}

static int zx_ioctl_2d_draw_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t                       *zx  = priv->card;
    void __user                      *argp = (void __user*)arg;

    zx_2d_draw_compat_t             draw_2d_compat;
    zx_2d_draw_t                    draw_2d;

    if(zx_copy_from_user(&draw_2d_compat, argp, sizeof(zx_2d_draw_compat_t)))
    {
        return -1;
    }
    
    draw_2d.len = draw_2d_compat.len;
    draw_2d.hSrcAllocation = draw_2d_compat.hSrcAllocation;
    draw_2d.hDstAllocation = draw_2d_compat.hDstAllocation;
    draw_2d.hMaskAllocation = draw_2d_compat.hMaskAllocation;
    draw_2d.kernel_mode_command = draw_2d_compat.kernel_mode_command;
    draw_2d.cmd = compat_ptr(draw_2d_compat.cmd);

    if( draw_2d.len > 0 )
    {
        draw_2d.kernel_mode_command = 0;
        krnl_kickoff_2d_draw(zx->adapter, &draw_2d);
    }

    return 0;
}

static int zx_ioctl_lock_allocation_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t        *zx  = priv->card;
    void __user       *argp = (void __user*)arg;

    zx_lock_allocation_compat_t lock_data_compat;
    zx_lock_allocation_t lock_data;
    int                   ret;

    if(zx_copy_from_user(&lock_data_compat, argp, sizeof(zx_lock_allocation_compat_t)))
    {
        return -1;
    }

    lock_data.device = lock_data_compat.device;
    lock_data.allocation = lock_data_compat.allocation;
    lock_data.acquire_apeture = lock_data_compat.acquire_apeture;
    lock_data.read_only = lock_data_compat.read_only;
    lock_data.write_only = lock_data_compat.write_only;
    lock_data.cache_type = lock_data_compat.cache_type;
    lock_data.virt_addr = compat_ptr(lock_data_compat.virt_addr);

    ret = krnl_lock_allocation(zx->adapter, &lock_data, ZX_MEM_USER);

    lock_data_compat.virt_addr = ptr_to_compat(lock_data.virt_addr);

    if(zx_copy_to_user(argp, &lock_data_compat, sizeof(zx_lock_allocation_t)))
    {
        return -1;
    }

    return ret;
}

static int zx_ioctl_get_address_config_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t  *zx  = priv->card;
    void __user *argp = (void __user*)arg;

    zx_get_address_config_t address_config;
    compat_uptr_t address_config_compat;

    krnl_get_address_config(zx->adapter, &address_config);

    address_config_compat = ulong_to_compat(address_config.mmioPhyAddr);

    if(zx_copy_to_user(argp, &address_config_compat, sizeof(address_config_compat)))
    {
        return -1;
    }

    return 0;
}

static int zx_ioctl_get_perf_event_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t   *zx = priv->card;
    void __user  *argp = (void __user*)arg;
    zx_get_perf_event_compat_t get_perf_event_compat = {0};
    zx_get_perf_event_t get_perf_event = {0};
    int          ret = 0;

    if (zx_copy_from_user(&get_perf_event_compat, argp, sizeof(zx_get_perf_event_compat_t)))
    {
        return -1;
    }

    get_perf_event.max_event_num = get_perf_event_compat.max_event_num;
    get_perf_event.max_event_buffer_size = get_perf_event_compat.max_event_buffer_size;
    get_perf_event.event_buffer = compat_ptr(get_perf_event_compat.event_buffer);
    get_perf_event.event_filled_num = get_perf_event_compat.event_filled_num;
    get_perf_event.event_buffer_filled_size = get_perf_event_compat.event_buffer_filled_size;
    get_perf_event.event_lost_num = get_perf_event_compat.event_lost_num;
    get_perf_event.event_lost_timestamp = get_perf_event_compat.event_lost_timestamp;
    get_perf_event.isr_event_buffer = compat_ptr(get_perf_event_compat.isr_event_buffer);
    get_perf_event.isr_filled_num = get_perf_event_compat.isr_filled_num;
    get_perf_event.isr_filled_size = get_perf_event_compat.isr_filled_size;

    ret = krnl_get_perf_event(zx->adapter, &get_perf_event);

    get_perf_event_compat.max_event_num = get_perf_event.max_event_num;
    get_perf_event_compat.max_event_buffer_size = get_perf_event.max_event_buffer_size;
    get_perf_event_compat.event_buffer = ptr_to_compat(get_perf_event.event_buffer);
    get_perf_event_compat.event_filled_num = get_perf_event.event_filled_num;
    get_perf_event_compat.event_buffer_filled_size = get_perf_event.event_buffer_filled_size;
    get_perf_event_compat.event_lost_num = get_perf_event.event_lost_num;
    get_perf_event_compat.event_lost_timestamp = get_perf_event.event_lost_timestamp;
    get_perf_event_compat.isr_event_buffer = ptr_to_compat(get_perf_event.isr_event_buffer);
    get_perf_event_compat.isr_filled_num = get_perf_event.isr_filled_num;
    get_perf_event_compat.isr_filled_size = get_perf_event.isr_filled_size;

    if (ret == 0)
    {
        zx_copy_to_user(argp, &get_perf_event_compat, sizeof(zx_get_perf_event_compat_t));
    }

    return ret;
}

static int zx_ioctl_set_miu_reg_list_perf_event_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t *zx = priv->card;
    void __user *argp = (void __user*)arg;
    zx_miu_reg_list_perf_event_compat_t miu_reg_list_compat = {0};
    zx_miu_reg_list_perf_event_t miu_reg_list = {0};
    int ret = 0;

    if(zx_copy_from_user(&miu_reg_list_compat, argp, sizeof(zx_miu_reg_list_perf_event_compat_t)))
    {
        return -1;
    }

    miu_reg_list.miu_table_length = miu_reg_list_compat.miu_table_length;
    miu_reg_list.miu_table = compat_ptr(miu_reg_list_compat.miu_table);

    ret = krnl_set_miu_reg_list_perf_event(zx->adapter, &miu_reg_list);

    return ret;
}

static int zx_ioctl_get_miu_dump_perf_event_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t *zx = priv->card;
    void __user *argp = (void __user*)arg;
    zx_get_miu_dump_perf_event_compat_t get_miu_dump_compat = {0};
    zx_get_miu_dump_perf_event_t get_miu_dump = {0};
    int ret = 0;

    if(zx_copy_from_user(&get_miu_dump_compat, argp, sizeof(zx_get_miu_dump_perf_event_compat_t)))
    {
        return -1;
    }

    get_miu_dump.max_event_num = get_miu_dump_compat.max_event_num;
    get_miu_dump.max_event_buffer_size = get_miu_dump_compat.max_event_buffer_size;
    get_miu_dump.event_buffer = compat_ptr(get_miu_dump_compat.event_buffer);
    get_miu_dump.event_filled_num = get_miu_dump_compat.event_filled_num;
    get_miu_dump.event_buffer_filled_size = get_miu_dump_compat.event_buffer_filled_size;
    get_miu_dump.event_lost_num = get_miu_dump_compat.event_lost_num;
    get_miu_dump.event_lost_timestamp = get_miu_dump_compat.event_lost_timestamp;

    ret = krnl_get_miu_dump_perf_event(zx->adapter, &get_miu_dump);

    get_miu_dump_compat.max_event_num = get_miu_dump.max_event_num;
    get_miu_dump_compat.max_event_buffer_size = get_miu_dump.max_event_buffer_size;
    get_miu_dump_compat.event_buffer = ptr_to_compat(get_miu_dump.event_buffer);
    get_miu_dump_compat.event_filled_num = get_miu_dump.event_filled_num;
    get_miu_dump_compat.event_buffer_filled_size = get_miu_dump.event_buffer_filled_size;
    get_miu_dump_compat.event_lost_num = get_miu_dump.event_lost_num;
    get_miu_dump_compat.event_lost_timestamp = get_miu_dump.event_lost_timestamp;

    if (ret == 0)
    {
        zx_copy_to_user(argp, &get_miu_dump_compat, sizeof(zx_get_miu_dump_perf_event_compat_t));
    }

    return ret;
}

static int zx_ioctl_direct_get_miu_dump_perf_event_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t *zx = priv->card;
    void __user *argp = (void __user*)arg;
    zx_direct_get_miu_dump_perf_event_compat_t direct_get_dump_compat = {0};
    zx_direct_get_miu_dump_perf_event_t direct_get_dump = {0};
    int ret = 0;

    if(zx_copy_from_user(&direct_get_dump_compat, argp, sizeof(zx_direct_get_miu_dump_perf_event_compat_t)))
    {
        return -1;
    }

    direct_get_dump.miu_table = compat_ptr(direct_get_dump_compat.miu_table);
    direct_get_dump.miu_table_length = direct_get_dump_compat.miu_table_length;
    direct_get_dump.timestamp_high = direct_get_dump_compat.timestamp_high;
    direct_get_dump.timestamp_low = direct_get_dump_compat.timestamp_low;

    ret = krnl_direct_get_miu_dump_perf_event(zx->adapter, &direct_get_dump);

    direct_get_dump_compat.miu_table = ptr_to_compat(direct_get_dump.miu_table);
    direct_get_dump_compat.miu_table_length = direct_get_dump.miu_table_length;
    direct_get_dump_compat.timestamp_high = direct_get_dump.timestamp_high;
    direct_get_dump_compat.timestamp_low = direct_get_dump.timestamp_low;
    
    if(ret == 0)
    {
        zx_copy_to_user(argp, &direct_get_dump_compat, sizeof(zx_direct_get_miu_dump_perf_event_compat_t));
    }

    return ret;
}

static int zx_ioctl_create_context_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t          *zx  = priv->card;
    void __user         *argp = (void __user*)arg;

    zx_create_context_compat_t create_context_compat = {0};
    zx_create_context_t create_context = {0};
    int                  ret  = 0;

    if(zx_copy_from_user(&create_context_compat, argp, sizeof(zx_create_context_compat_t)))
    {
        return -1;
    }

    create_context.device = create_context_compat.device;
    create_context.engine_index = create_context_compat.engine_index;
    create_context.flags = create_context_compat.flags;
    create_context.context = create_context_compat.context;
    create_context.command_buffer = compat_ptr(create_context_compat.command_buffer);
    create_context.command_buffer_size = create_context_compat.command_buffer_size;
    create_context.allocation_list = compat_ptr(create_context_compat.allocation_list);
    create_context.allocation_list_size = create_context_compat.allocation_list_size;
    create_context.patch_location_list = compat_ptr(create_context_compat.patch_location_list);
    create_context.patch_location_list_size = create_context_compat.patch_location_list_size;
    create_context.sync_object_list = compat_ptr(create_context_compat.sync_object_list);
    create_context.sync_object_list_size = create_context_compat.sync_object_list_size;

    ret = krnl_create_context(zx->adapter, &create_context, ZX_MEM_USER);

    create_context_compat.device = create_context.device;
    create_context_compat.engine_index = create_context.engine_index;
    create_context_compat.flags = create_context.flags;
    create_context_compat.context = create_context.context;
    create_context_compat.command_buffer = ptr_to_compat(create_context.command_buffer);
    create_context_compat.command_buffer_size = create_context.command_buffer_size;
    create_context_compat.allocation_list = ptr_to_compat(create_context.allocation_list);
    create_context_compat.allocation_list_size = create_context.allocation_list_size;
    create_context_compat.patch_location_list = ptr_to_compat(create_context.patch_location_list);
    create_context_compat.patch_location_list_size = create_context.patch_location_list_size;
    create_context_compat.sync_object_list = ptr_to_compat(create_context.sync_object_list);
    create_context_compat.sync_object_list_size = create_context.sync_object_list_size;

    if (ret != S_OK)
    {
        return -1;
    }

    if(zx_copy_to_user(argp, &create_context_compat, sizeof(zx_create_context_compat_t)))
    {
        return -1;
    }

    return ret;
}

static int zx_ioctl_render_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t   *zx    = priv->card;
    void __user  *argp   = (void __user*)arg;

    zx_render_compat_t  render_compat = {0};
    zx_render_t  render = {0};
    int           ret    = 0;

    if(zx_copy_from_user(&render_compat, argp, sizeof(zx_render_compat_t)))
    {
        return -1;
    }

    render.context = render_compat.context;
    render.command_length = render_compat.command_length;
    render.command_offset = render_compat.command_offset;
    render.allocation_count = render_compat.allocation_count;
    render.patch_location_count = render_compat.patch_location_count;
    render.sync_object_count = render_compat.sync_object_count;
    render.new_command_buffer = compat_ptr(render_compat.new_command_buffer);
    render.new_command_buffer_size = render_compat.new_command_buffer_size;
    render.new_allocation_list = compat_ptr(render_compat.new_allocation_list);
    render.new_allocation_list_size = render_compat.new_allocation_list_size;
    render.new_patch_location_list = compat_ptr(render_compat.new_patch_location_list);
    render.new_patch_location_list_size = render_compat.new_patch_location_list_size;
    render.flags = render_compat.flags;

    ret = krnl_render(zx->adapter, &render);

    render_compat.context = render.context;
    render_compat.command_length = render.command_length;
    render_compat.command_offset = render.command_offset;
    render_compat.allocation_count = render.allocation_count;
    render_compat.patch_location_count = render.patch_location_count;
    render_compat.sync_object_count = render.sync_object_count;
    render_compat.new_command_buffer = ptr_to_compat(render.new_command_buffer);
    render_compat.new_command_buffer_size = render.new_command_buffer_size;
    render_compat.new_allocation_list = ptr_to_compat(render.new_allocation_list);
    render_compat.new_allocation_list_size = render.new_allocation_list_size;
    render_compat.new_patch_location_list = ptr_to_compat(render.new_patch_location_list);
    render_compat.new_patch_location_list_size = render.new_patch_location_list_size;
    render_compat.flags = render.flags;

    if(zx_copy_to_user(argp, &render_compat, sizeof(zx_render_compat_t)))
    {
        return -1;
    }

    return ret;
}

static int zx_ioctl_ipc_operation_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t              *zx  = priv->card;
    void __user             *argp = (void __user*)arg;
    int                     ret=0;

    zx_ipc_operation_compat_t     ipc_operation_compat;
    zx_ipc_operation_t     ipc_operation;

    if(zx_copy_from_user(&ipc_operation_compat, argp, sizeof(zx_ipc_operation_compat_t)))
    {
        return -1;
    }

    ipc_operation.type = ipc_operation_compat.type;
    ipc_operation.operation = ipc_operation_compat.operation; 
    ipc_operation.timeout = ipc_operation_compat.timeout; 
    ipc_operation.size_in = ipc_operation_compat.size_in; 
    ipc_operation.size_out = ipc_operation_compat.size_out; 
    ipc_operation.send_data = compat_ptr(ipc_operation_compat.send_data); 
    ipc_operation.reply_data = compat_ptr(ipc_operation_compat.reply_data); 

    ret = zx_ipc_operation(zx, priv, &ipc_operation);

    return ret;
}

static int zx_ioctl_send_dsi_cmd_compat(zx_file_t *priv, unsigned int cmd, unsigned long arg)
{
    zx_card_t      *zx  = priv->card;
    void __user     *argp = (void __user*)arg;
    zx_dsi_cmd_compat_t dsi_cmd_compat;
    zx_dsi_cmd_t dsi_cmd;
    int ret = 0;

    if(zx_copy_from_user(&dsi_cmd_compat, argp, sizeof(zx_dsi_cmd_compat_t)))
    {
        return -1;
    }

    dsi_cmd.dsi_opcode = dsi_cmd_compat.dsi_opcode;
    switch(dsi_cmd.dsi_opcode)
    {
        case ZX_DSI_READ_CMD:
            dsi_cmd.read_cmd.dsi_index = dsi_cmd_compat.read_cmd.dsi_index;
            dsi_cmd.read_cmd.virtual_ch = dsi_cmd_compat.read_cmd.virtual_ch;
            dsi_cmd.read_cmd.content_type = dsi_cmd_compat.read_cmd.content_type;
            dsi_cmd.read_cmd.data_len = dsi_cmd_compat.read_cmd.data_len;
            dsi_cmd.read_cmd.data_buf = compat_ptr(dsi_cmd_compat.read_cmd.data_buf);
            zx_memcpy(dsi_cmd.read_cmd.received_payload, dsi_cmd_compat.read_cmd.received_payload, 16);
            dsi_cmd.read_cmd.payload_len = dsi_cmd_compat.read_cmd.payload_len;
            dsi_cmd.read_cmd.is_HSmode_only = dsi_cmd_compat.read_cmd.is_HSmode_only;
            break;
        case ZX_DSI_WRITE_CMD:
            dsi_cmd.write_cmd.dsi_index = dsi_cmd_compat.write_cmd.dsi_index;
            dsi_cmd.write_cmd.virtual_ch = dsi_cmd_compat.write_cmd.virtual_ch;
            dsi_cmd.write_cmd.packet_type = dsi_cmd_compat.write_cmd.packet_type;
            dsi_cmd.write_cmd.content_type = dsi_cmd_compat.write_cmd.content_type;
            dsi_cmd.write_cmd.data_len = dsi_cmd_compat.write_cmd.data_len;
            dsi_cmd.write_cmd.data_buf = compat_ptr(dsi_cmd_compat.write_cmd.data_buf);
            dsi_cmd.write_cmd.is_need_ack = dsi_cmd_compat.write_cmd.is_need_ack;
            dsi_cmd.write_cmd.is_HSmode_only = dsi_cmd_compat.write_cmd.is_HSmode_only;
            break;
        default:
            break;
    }

    ret = krnl_send_dsi_cmd(zx->adapter, &dsi_cmd);

    switch(dsi_cmd.dsi_opcode)
    {
        case ZX_DSI_READ_CMD:
            dsi_cmd_compat.read_cmd.dsi_index = dsi_cmd.read_cmd.dsi_index;
            dsi_cmd_compat.read_cmd.virtual_ch = dsi_cmd.read_cmd.virtual_ch;
            dsi_cmd_compat.read_cmd.content_type = dsi_cmd.read_cmd.content_type;
            dsi_cmd_compat.read_cmd.data_len = dsi_cmd.read_cmd.data_len;
            dsi_cmd_compat.read_cmd.data_buf = ptr_to_compat(dsi_cmd.read_cmd.data_buf);
            zx_memcpy(dsi_cmd_compat.read_cmd.received_payload, dsi_cmd.read_cmd.received_payload, 16);
            dsi_cmd_compat.read_cmd.payload_len = dsi_cmd.read_cmd.payload_len;
            dsi_cmd_compat.read_cmd.is_HSmode_only = dsi_cmd.read_cmd.is_HSmode_only;
            break;
        case ZX_DSI_WRITE_CMD:
            dsi_cmd_compat.write_cmd.dsi_index = dsi_cmd.write_cmd.dsi_index;
            dsi_cmd_compat.write_cmd.virtual_ch = dsi_cmd.write_cmd.virtual_ch;
            dsi_cmd_compat.write_cmd.packet_type = dsi_cmd.write_cmd.packet_type;
            dsi_cmd_compat.write_cmd.content_type = dsi_cmd.write_cmd.content_type;
            dsi_cmd_compat.write_cmd.data_len = dsi_cmd.write_cmd.data_len;
            dsi_cmd_compat.write_cmd.data_buf = ptr_to_compat(dsi_cmd.write_cmd.data_buf);
            dsi_cmd_compat.write_cmd.is_need_ack = dsi_cmd.write_cmd.is_need_ack;
            dsi_cmd_compat.write_cmd.is_HSmode_only = dsi_cmd.write_cmd.is_HSmode_only;
            break;
        default:
            break;
    }

    if(zx_copy_to_user(argp, &dsi_cmd, sizeof(zx_dsi_cmd_t)))
    {
        return -1;
    }

    return ret;
}

zx_ioctl_t zx_ioctls_compat[] =
{
    [ZX_IOCTL_NR(ZX_IOCTL_GET_BUSID)]                = zx_ioctl_get_busid,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_DISPLAYINFO)]          = zx_ioctl_get_display_info,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_IGAINFO)]              = zx_ioctl_get_iga_info,
    [ZX_IOCTL_NR(ZX_IOCTL_ADD_CTX)]                  = zx_ioctl_addctx,
    [ZX_IOCTL_NR(ZX_IOCTL_RM_CTX)]                   = zx_ioctl_rmctx,
    [ZX_IOCTL_NR(ZX_IOCTL_LOCK)]                     = zx_ioctl_lock_ctx,
    [ZX_IOCTL_NR(ZX_IOCTL_UNLOCK)]                   = zx_ioctl_unlock_ctx,
    [ZX_IOCTL_NR(ZX_IOCTL_CLIP_NOTIFY)]              = zx_ioctl_clip_notify_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_ADD_DRAW)]                 = zx_ioctl_add_drawable_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_RM_DRAW)]                  = zx_ioctl_remove_drawable,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_DRAWSIZE)]             = zx_ioctl_get_drawsize,
    [ZX_IOCTL_NR(ZX_IOCTL_QUERY_DISPLAY_CAPS)]       = zx_ioctl_query_display_caps,
    [ZX_IOCTL_NR(ZX_IOCTL_QUERY_OUTPUT_CAPS)]        = zx_ioctl_query_output_caps_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_MODE_SET)]                 = zx_ioctl_set_mode,
    [ZX_IOCTL_NR(ZX_IOCTL_CURSOR)]                   = zx_ioctl_cursor,
    [ZX_IOCTL_NR(ZX_IOCTL_GAMMA)]                    = zx_ioctl_gamma_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_DPMS)]                     = zx_ioctl_dpms,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_MODE_TIMING)]          = zx_ioctl_get_mode_timing,
    [ZX_IOCTL_NR(ZX_IOCTL_2D_INIT)]                  = zx_ioctl_2d_init,
    [ZX_IOCTL_NR(ZX_IOCTL_INIT_SCREEN)]              = zx_ioctl_init_screen_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_DEINIT_SCREEN)]            = zx_ioctl_deinit_screen_compat,    
    [ZX_IOCTL_NR(ZX_IOCTL_WAIT_CHIP_IDLE)]           = zx_ioctl_wait_chip_idle,
    [ZX_IOCTL_NR(ZX_IOCTL_ESCAPE_CALL)]              = zx_ioctl_escape_call_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_2D_DRAW)]                  = zx_ioctl_2d_draw_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_ACQUIRE_APERTURE)]         = zx_ioctl_acquire_aperture,
    [ZX_IOCTL_NR(ZX_IOCTL_RELEASE_APERTURE)]         = zx_ioctl_release_aperture,
    [ZX_IOCTL_NR(ZX_IOCTL_LEAVE_VT)]                 = zx_ioctl_leave_vt,
    [ZX_IOCTL_NR(ZX_IOCTL_ENTER_VT)]                 = zx_ioctl_enter_vt,
    [ZX_IOCTL_NR(ZX_IOCTL_CHECK_DEVICE_COMBINATION)] = zx_ioctl_check_device_combination,
    [ZX_IOCTL_NR(ZX_IOCTL_SET_PS)]                   = zx_ioctl_set_ps,
    [ZX_IOCTL_NR(ZX_IOCTL_ENUM_SUB_DEVICE)]          = zx_ioctl_enum_sub_device,
    [ZX_IOCTL_NR(ZX_IOCTL_CREATE_ALLOCATION)]        = zx_ioctl_create_allocation,
    [ZX_IOCTL_NR(ZX_IOCTL_OPEN_ALLOCATION)]          = zx_ioctl_open_allocation,
    [ZX_IOCTL_NR(ZX_IOCTL_DESTROY_ALLOCATION)]       = zx_ioctl_destroy_allocation,
    [ZX_IOCTL_NR(ZX_IOCTL_LOCK_ALLOCATION)]          = zx_ioctl_lock_allocation_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_UNLOCK_ALLOCATION)]        = zx_ioctl_unlock_allocation,
    [ZX_IOCTL_NR(ZX_IOCTL_FLUSH_ALLOCATION)]         = zx_ioctl_flush_allocation,
    [ZX_IOCTL_NR(ZX_IOCTL_WAIT_ALLOCATION_IDLE)]     = zx_ioctl_wait_allocation_idle,
    [ZX_IOCTL_NR(ZX_IOCTL_VIDEO_GET_HW_CAPS)]        = zx_ioctl_video_get_hw_caps,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_ADDRESS_CONFIG)]       = zx_ioctl_get_address_config_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_QUERY_INFO)]               = zx_ioctl_query_info,
    [ZX_IOCTL_NR(ZX_IOCTL_WAIT_FUNC)]                = zx_ioctl_wait_func,
    [ZX_IOCTL_NR(ZX_IOCTL_FLIP)]                     = zx_ioctl_flip,
    [ZX_IOCTL_NR(ZX_IOCTL_CEC_CTL)]                  = zx_ioctl_cec_ctl,
    [ZX_IOCTL_NR(ZX_IOCTL_PRESENT)]                  = zx_ioctl_present,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_DRAWABLE_SURFACE)]     = zx_ioctl_get_drawable_surface,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_VSYNC_INFO)]           = zx_ioctl_get_vsync_info,
    [ZX_IOCTL_NR(ZX_IOCTL_CREATE_DEVICE)]            = zx_ioctl_create_device,
    [ZX_IOCTL_NR(ZX_IOCTL_DESTROY_DEVICE)]           = zx_ioctl_destroy_device,
    [ZX_IOCTL_NR(ZX_IOCTL_BEGIN_PERF_EVENT)]         = zx_ioctl_begin_perf_event,
    [ZX_IOCTL_NR(ZX_IOCTL_END_PERF_EVENT)]           = zx_ioctl_end_perf_event,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_PERF_EVENT)]           = zx_ioctl_get_perf_event_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_SEND_PERF_EVENT)]          = zx_ioctl_send_perf_event,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_PERF_STATUS)]          = zx_ioctl_get_perf_status,
    [ZX_IOCTL_NR(ZX_IOCTL_BEGIN_MIU_DUMP_PERF_EVENT)] = zx_ioctl_begin_miu_dump_perf_event,
    [ZX_IOCTL_NR(ZX_IOCTL_END_MIU_DUMP_PERF_EVENT)]  = zx_ioctl_end_miu_dump_perf_event,
    [ZX_IOCTL_NR(ZX_IOCTL_SET_MIU_REG_LIST_PERF_EVENT)] = zx_ioctl_set_miu_reg_list_perf_event_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_MIU_DUMP_PERF_EVENT)]  = zx_ioctl_get_miu_dump_perf_event_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_DIRECT_GET_MIU_DUMP_PERF_EVENT)] = zx_ioctl_direct_get_miu_dump_perf_event_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_CREATE_CONTEXT)]           = zx_ioctl_create_context_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_DESTROY_CONTEXT)]          = zx_ioctl_destroy_context,
    [ZX_IOCTL_NR(ZX_IOCTL_WAIT_CONTEXT_IDLE)]        = zx_ioctl_wait_context_idle,
    [ZX_IOCTL_NR(ZX_IOCTL_RENDER)]                   = zx_ioctl_render_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_QUERY_CHIP_ID)]            = zx_ioctl_query_chip_id,
    [ZX_IOCTL_NR(ZX_IOCTL_HDCP_OP)]                  = zx_ioctl_hdcp_op,
    [ZX_IOCTL_NR(ZX_IOCTL_CREATE_FENCE_SYNC_OBJECT)] = zx_ioctl_create_fence_sync_object,
    [ZX_IOCTL_NR(ZX_IOCTL_DESTROY_FENCE_SYNC_OBJECT)]= zx_ioctl_destroy_fence_sync_object,
    [ZX_IOCTL_NR(ZX_IOCTL_WAIT_FENCE_SYNC_OBJECT)]   = zx_ioctl_wait_fence_sync_object,
    [ZX_IOCTL_NR(ZX_IOCTL_FENCE_VALUE)]              = zx_ioctl_fence_value,
    [ZX_IOCTL_NR(ZX_IOCTL_CREATE_FENCE_FD)]          = zx_ioctl_create_fence_fd,
    [ZX_IOCTL_NR(ZX_IOCTL_SET_FLIP_COUNTER)]         = zx_ioctl_flip_limit,
    [ZX_IOCTL_NR(ZX_IOCTL_IPC_OPERATION)]            = zx_ioctl_ipc_operation_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_CREATE_STREAM)]            = zx_ioctl_create_stream,
    [ZX_IOCTL_NR(ZX_IOCTL_DESTROY_STREAM)]           = zx_ioctl_destroy_stream,
    [ZX_IOCTL_NR(ZX_IOCTL_DISPLAY_FRAME)]            = zx_ioctl_display_frame,
    [ZX_IOCTL_NR(ZX_IOCTL_SET_INTERACTIVE)]          = zx_ioctl_set_interactive,
    [ZX_IOCTL_NR(ZX_IOCTL_SEND_DSI_CMD)]             = zx_ioctl_send_dsi_cmd_compat,
    [ZX_IOCTL_NR(ZX_IOCTL_MHL_CBUS_SEND_CMD)]        = zx_ioctl_mhl_cbus_send_command,
    [ZX_IOCTL_NR(ZX_IOCTL_CREATE_DI_CONTEXT)]        = zx_ioctl_create_di_context,
    [ZX_IOCTL_NR(ZX_IOCTL_DESTROY_DI_CONTEXT)]       = zx_ioctl_destroy_di_context,
    [ZX_IOCTL_NR(ZX_IOCTL_REGISTER_MHL_CALLBACK)]    = zx_ioctl_register_mhl_callback,
    [ZX_IOCTL_NR(ZX_IOCTL_SET_POWER_PERF_MODE)]      = zx_ioctl_set_power_perf_mode,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_DEVICE_COMB)]          = zx_ioctl_get_device_comb,
    [ZX_IOCTL_NR(ZX_IOCTL_GFX_STM)]                  = zx_ioctl_gfx_stm,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_HDMI_AUDIO_FORMAT)]    = zx_ioctl_get_hdmi_audio_format,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_HPD_INFO)]             = zx_ioctl_get_hpd_info,
    [ZX_IOCTL_NR(ZX_IOCTL_HANDLE_TO_FD)]             = zx_ioctl_handle_to_fd,
    [ZX_IOCTL_NR(ZX_IOCTL_FD_TO_HANDLE)]             = zx_ioctl_fd_to_handle,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_UST_MSC)]              = zx_ioctl_get_ust_msc,
    [ZX_IOCTL_NR(ZX_IOCTL_GET_ALLOCATION_STATE)]     = zx_ioctl_get_allocation_state,
    [ZX_IOCTL_NR(ZX_IOCTL_WAIT_EVENT)]               = zx_ioctl_wait_event,
};

