/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __ZX_SHARED_H__
#define __ZX_SHARED_H__

#include "zx_def.h"
#include "zx_driver.h"

typedef struct framebuffer
{
    int server_index;
    int screen_index;
    int master;
    int samm;
    int mamm;
    int duoview;
    int use_rxa;
    int width;
    int height;
    int depth;
    int tiled;
    unsigned int hdevice;
    unsigned int hAllocation;
    unsigned int rot_allocation[MAX_CORE_CRTCS];
    void *adapter;
    int pitch;
    int gbd;
    unsigned long fbOffset;
    void *priv;
}framebuffer_t;

typedef struct  
{
    int                          xinerama;

    framebuffer_t                *frame_buffers[ZX_MAX_CARDS][MAX_SCREENS];
    framebuffer_t                *frame_shadow_buffers[ZX_MAX_CARDS][MAX_SCREENS];
    
}server_info_t;

extern void zx_init_2d_screen(zx_card_t *card, zx_file_t *priv, zx_2d_screen_t *screen);
extern void zx_deinit_2d_screen(zx_card_t *card, zx_2d_screen_t *screen);
extern int  zx_is_xinerama(zx_card_t *card, int server_index);
extern void zx_set_server_info(zx_card_t *card, int server_index, int xinerama);
extern int zx_get_display_info(zx_card_t *card, zx_display_info_t *display_info);
extern void zx_clip_notify(zx_card_t *card, zx_clipnotify_t *clip);
extern void zx_add_drawable(zx_card_t *card, zx_drawable_t *drawable);
extern void zx_get_drawable_info(zx_card_t *card, zx_get_drawsize_t *draw_rec);
extern void zx_remove_drawable(zx_card_t *card, int drawId);
extern framebuffer_t* zx_get_frame_buffer(zx_card_t *card, int server_index, int screen_index);
extern void zx_map_iga_to_screen(zx_card_t *card, int *screens, int server_index);
extern void zx_map_screen_to_igas(zx_card_t *card, int *igas, int server_index, int screen_index);
extern zx_drawable_t *zx_get_drawable(unsigned int kdraw);
extern void zx_set_current_fb(zx_card_t *card, int server_index, int screen_index, framebuffer_t *fb);
extern void zx_dump_clips(zx_card_t *card);
extern void zx_video_drawable_damage(zx_card_t *card, zx_video_damage_t *damage);
#endif
