/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/*
* ZX SDK TYPES
*/
#ifndef __ZX_TYPES_H__
#define __ZX_TYPES_H__


typedef enum
{
    ZX_USAGE_DISPLAY_SURFACE   = 0x00000001,
    ZX_USAGE_OVERLAY_SURFACE   = 0x00000002,
    ZX_USAGE_CURSOR_SURFACE    = 0x00000004,
    ZX_USAGE_RENDER_TARGET     = 0x00000008,
    ZX_USAGE_TEXTURE_BUFFER    = 0x00000010,
    ZX_USAGE_SHADER_BUFFER     = 0x00000020,
    ZX_USAGE_DATA_BUFFER       = 0x00000040,
    ZX_USAGE_TEMP_BUFFER       = 0x00000080,
    ZX_USAGE_COMPOSER_SURFACE  = 0x00000100,
    ZX_USAGE_SHADOW_SURFACE    = 0x00000200,
    ZX_USAGE_VIDEO             = 0x00000400,
    ZX_USAGE_VIDEO_DECODER     = 0x00000400,
    ZX_USAGE_CAMERA            = 0x00000800,
    ZX_USAGE_VIDEO_ENCODER     = 0x00001000,
    ZX_USAGE_RENDER_SCRIPT     = 0x00002000,

    ZX_USAGE_CREATE_PIXMAP     = 0x01000000,

    ZX_USAGE_FRAMEBUFFER       = 0x08000000,
    ZX_USAGE_TRY_TILED         = 0x80000000,
}zx_usage_bit;

typedef enum
{
    ZX_ACCESS_UNKNOWN    = 0,
    ZX_ACCESS_GPU_ONLY   = 1,
    ZX_ACCESS_GPU_ALMOST,
    ZX_ACCESS_CPU_ALMOST,
//    ZX_ACCESS_CPU_ONLY,
}zx_access_hint;

typedef enum
{
    ZX_FORMAT_A8_UNORM       = 1,
    ZX_FORMAT_B5G6R5_UNORM   = 2,        /*rmask 0xf800; gmask 0x7e0; bmask 0x1f*/
    ZX_FORMAT_B5G5R5A1_UNORM = 3,
    ZX_FORMAT_A1B5G5R5_UNORM = 4,
    ZX_FORMAT_B4G4R4A4_UNORM = 5,
    ZX_FORMAT_A4B4G4R4_UNORM = 6,
    ZX_FORMAT_B8G8R8A8_UNORM = 7,      /*rmask 0x00ff0000; gmask 0x0000ff00; bmask 0x000000ff; amask 0xff000000*/
    ZX_FORMAT_B8G8R8X8_UNORM = 8,
    ZX_FORMAT_R8G8B8A8_UNORM = 9,      /*rmask 0x000000ff; gmask 0x0000ff00; bmask 0x00ff0000; amask 0xff000000*/
    ZX_FORMAT_R8G8B8X8_UNORM = 10,
    ZX_FORMAT_A8R8G8B8_UNORM = 11,
    ZX_FORMAT_YUY2           = 12,
    ZX_FORMAT_NV12_LINEAR    = 13,
    ZX_FORMAT_NV12_TILED     = 14,
    ZX_FORMAT_NV21_LINEAR    = 15,
    ZX_FORMAT_YV12           = 16,
    ZX_FORMAT_FLOAT32        = 17,
    ZX_FORMAT_UINT32         = 18,
    ZX_FORMAT_INT32          = 19,
    ZX_FORMAT_R8_UNORM       = 20,

    ZX_FORMAT_RAW10          = 21,
    ZX_FORMAT_RAW16          = 22,
    ZX_FORMAT_RAW_OPAQUE     = 23,
    ZX_FORMAT_B10G10R10A2_UNORM = 24,
}zx_format;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int width;             /* in */
    unsigned int height;            /* in */
    unsigned int usage_mask;        /* in */
    zx_format   format;            /* in */
    zx_access_hint access_hint;    /* in */
    unsigned int fence_sync  : 1;   /* in */
    unsigned int need_fd     : 1;   /* in */
    unsigned int unpagable   : 1;   /* in/out */
    unsigned int tiled       : 1;   /* in/out */
    unsigned int secured     : 1;   /* in/out */
    unsigned int lcs         : 2;   /* in/out */
    unsigned int compressed  : 1;   /* in/out */
    unsigned int share       : 1;   /* in/out */
    unsigned int primary     : 1;   /* in/out */

    unsigned int hw_format;         /* out */
    unsigned int size;              /* out */
    unsigned int width_aligned;     /* out */
    unsigned int height_aligned;    /* out */
    unsigned int bit_cnt;           /* out */
    unsigned int pitch;             /* out */
    unsigned int gbd;               /* out */
    unsigned int tiled_width;       /* out */
    unsigned int tiled_height;      /* out */
    unsigned int gpu_virt_addr;     /* out */
    unsigned int allocation;        /* out */
    unsigned int sync_obj;          /* out */
    unsigned int fence_addr;        /* out */
    unsigned int compress_format;   /* out */
    unsigned int buffer_fd;         /* out */
    unsigned int segment_id;        /* out */
    unsigned int bFolding;          /* out */
    unsigned int eBlockSize;        /* out */
    unsigned int FoldingDir;        /* out */
}zx_create_allocation_t;

typedef struct
{
    unsigned int hDevice;            /* in */
    unsigned int hAllocation;        /* in */
    unsigned int fill_pattern;       /* in */
}zx_fill_allocation_t;



typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */

    unsigned int size;              /* out */
    unsigned int width;             /* out */
    unsigned int height;            /* out */
    unsigned int aligned_width;     /* out */
    unsigned int aligned_height;    /* out */
    unsigned int tiled_width;       /* out */
    unsigned int tiled_height;      /* out */
    unsigned int bit_cnt;           /* out */
    unsigned int pitch;             /* out */
    unsigned int unpagable : 1;     /* out */
    unsigned int tiled     : 1;     /* out */
    unsigned int secured   : 1;     /* out */
    unsigned int snoop     : 1;     /* out */
    unsigned int local     : 1;     /* out */
    unsigned int ForceLocal: 1;     /* out */
    unsigned int dma_mapped :1;     /* out */

    unsigned int hw_format;         /* out */
    unsigned int segment_id;        /* out */
    unsigned int bFolding;          /* out */
    unsigned int eBlockSize;        /* out */
    unsigned int FoldingDir;        /* out */

    unsigned int gpu_virt_addr;     /* out */
    unsigned int gbd;               /* out */
    unsigned int sync_obj;          /* out */
    unsigned int fence_addr;        /* out */
}zx_open_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
}zx_destroy_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
    unsigned int acquire_apeture :1;/* in */
    unsigned int read_only       :1;/* in */
    unsigned int write_only      :1;/* in */
    unsigned int cache_type      :3;/* in */

    void         *virt_addr;        /* out */
}zx_lock_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
}zx_unlock_allocation_t;

typedef struct 
{
    unsigned int device;
    unsigned int allocation;
}zx_acquire_aperture_t;

typedef struct
{
    unsigned int device;
    unsigned int allocation;
    int          force_release;
}zx_release_aperture_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
    unsigned int width;             /* in */
    unsigned int height;            /* in */
    unsigned int width_aligned;     /* in */
    unsigned int height_aligned;    /* in */
    unsigned int pitch;             /* in */
    unsigned int acquire_apeture :1;/* in */
    unsigned int read_only       :1;/* in */
    unsigned int write_only      :1;/* in */

    void         *yaddr;            /* out */
    void         *uaddr;            /* out */
    void         *vaddr;            /* out */
}zx_lock_yv12_allocation_t;

typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
    unsigned int width;             /* in */
    unsigned int height;            /* in */
    unsigned int width_aligned;     /* in */
    unsigned int height_aligned;    /* in */
    unsigned int pitch;             /* in */
    unsigned int release_apeture :1;/* in */

    void         *yaddr;            /* in */
    void         *uaddr;            /* in */
    void         *vaddr;            /* in */
}zx_unlock_yv12_allocation_t;


typedef struct
{
    unsigned int device;            /* in */
    unsigned int allocation;        /* in */
    unsigned int offset;            /* in */
    unsigned int size;              /* in */
}zx_flush_allocation_t;


typedef struct
{
    unsigned int device;         /* out */
}zx_create_device_t;

typedef struct
{
    unsigned int  device;                   /* in   */
    unsigned int  engine_index;             /* in   */
    unsigned int  flags;                    /* in   */
    unsigned int  context;                  /* out  */
    unsigned int *command_buffer;           /* out  */
    unsigned int  command_buffer_size;      /* out  */
    void         *allocation_list;          /* out  */
    unsigned int  allocation_list_size;     /* out  */
    void         *patch_location_list;      /* out  */
    unsigned int  patch_location_list_size; /* out  */
    void         *sync_object_list;         /* out  */
    unsigned int  sync_object_list_size;    /* out  */
}zx_create_context_t;

typedef struct
{
    unsigned int device;              /* in   */
    unsigned int context;             /* in   */
}zx_destroy_context_t;

typedef struct
{
    unsigned int  device;              /* in   */
    unsigned int  stream_id;           /* in video stream id */
    unsigned int  context;             /* out  */
    unsigned int  hw_idx;              /* out  */
}zx_create_di_context_t;

typedef struct
{
    unsigned int device;              /* in   */
    unsigned int context;             /* in   */
}zx_destroy_di_context_t;

typedef struct
{
    unsigned int            context;                        /* in                   */
    unsigned int            command_length;                 /* in(number of bytes)  */
    unsigned int            command_offset;                 /* in                   */
    unsigned int            allocation_count;               /* in                   */
    unsigned int            patch_location_count;           /* in                   */
    unsigned int            sync_object_count;              /* in                   */
    void*                   new_command_buffer;             /* out                  */
    unsigned int            new_command_buffer_size;        /* in/out               */
    void*                   new_allocation_list;            /* out                  */
    unsigned int            new_allocation_list_size;       /* in/out               */
    void*                   new_patch_location_list;        /* out                  */
    unsigned int            new_patch_location_list_size;   /* in/out               */
    unsigned int            flags;                          /* in                   */
}zx_render_t;

/* ZX blending */

enum {
    /* no blending */
    ZX_BLENDING_NONE     = 0x00,

    /* ONE / ONE_MINUS_SRC_ALPHA */
    ZX_BLENDING_PREMULT  = 0x01,

    /* SRC_ALPHA / ONE_MINUS_SRC_ALPHA */
    ZX_BLENDING_COVERAGE = 0x02,

    /* ONE_MINUS_DST_ALPHA / DST_ALPHA */
    ZX_BLENDING_COVERAGE_INVERT = 0x03
};

/* ZX transform */
enum {
    ZX_TRANSFORM_NONE      = 0x00,
    /* flip source image horizontally (around the vertical axis) */
    ZX_TRANSFORM_FLIP_H    = 0x01,
    /* flip source image vertically (around the horizontal axis)*/
    ZX_TRANSFORM_FLIP_V    = 0x02,
    /* rotate source image 90 degrees counter clockwise */
    ZX_TRANSFORM_ROT_90    = 0x04,
    /* rotate source image 180 degrees */
    ZX_TRANSFORM_ROT_180   = 0x03,
    /* rotate source image 270 degrees counter clockwise */
    ZX_TRANSFORM_ROT_270   = 0x07,
    /* rotate source image 90 degrees counter clockwise, then do flip vertically */
    ZX_TRANSFORM_90_FLIP_H = 0x05,
    /* rotate source image 90 degrees counter clockwise, then do flip horizontally */
    ZX_TRANSFORM_90_FLIP_V = 0x06,
};





/**************fence sync object relate***************/
typedef struct
{
    unsigned int       device;                                /* in  */
    unsigned long long init_value;                            /* in  */
    unsigned int       fence_sync_object;                     /* out */
    unsigned int       fence_addr;                            /* out */
}zx_create_fence_sync_object_t;

typedef struct
{
    unsigned int       device;                   /* in */
    unsigned int       fence_sync_object;        /* in */
}zx_destroy_fence_sync_object_t;

#define ZX_SYNC_OBJ_ERROR                -1
#define ZX_SYNC_OBJ_INVALID_ARGU         -2
#define ZX_SYNC_OBJ_ALREAD_SIGNALED       1
#define ZX_SYNC_OBJ_TIMEOUT_EXPIRED       2
#define ZX_SYNC_OBJ_CONDITION_SATISFIED   3
#define ZX_SYNC_OBJ_WAIT_ON_SERVER        4
#define ZX_SYNC_OBJ_UNSIGNALED            5

typedef struct
{
    unsigned int       context;                  /* in */
    unsigned int       fence_sync_object;        /* in */
    unsigned int       server_wait;              /* in */
    unsigned long long timeout;                  /* in nano_sec */  //timeout == 0, no wait just return current object status
    unsigned long long fence_value;              /* in */
    int                status;                   /* out */
}zx_wait_fence_sync_object_t;

#define SVC_SET_FENCE_VALUE   0x01
#define SVC_GET_FENCE_VALUE   0x02

typedef struct
{
    unsigned int       device;                   /* in */
    unsigned int       opcode;                   /* in */
    unsigned int       fence_sync_object;        /* in */
    unsigned long long fence_value;              /* out */
    int                appid;                    /* out */
    int                flip_limit;               /* out */
    int                discard_sync;             /* in & out*/
}zx_fence_value_t;

typedef struct
{
    unsigned int       device;                   /* in  */
    unsigned int       context;                  /* in  */
    unsigned long long fence_value;              /* in  */
    unsigned int       fd;                       /* out */
    unsigned int       fence_addr;               /* out */
    unsigned int       sync_obj;                 /* out */
}zx_create_fence_fd_t;

typedef struct
{
    unsigned int       context;                  /* in */
    unsigned int       server_wait;              /* in */
    unsigned long long timeout;                  /* in nano_sec */  //timeout == 0, no wait just return current object status
    int                status;                   /* out */
}zx_wait_fence_t;

typedef struct
{
    unsigned int       sync_object;        /* out */
    unsigned long long fence_value;        /* out */
    unsigned int       fence_addr;         /* out */
}zx_fence_info_t;

typedef struct
{
    int fd2;
    char name[32];
    int fence;
}zx_fence_merge_t;

typedef struct
{
    int                fd;                  /* output for get, input for set */
}zx_bo_fence_t;

#define ZX_SLOT_SIZE     5
#define ZX_DATA_SLOT0    0
#define ZX_DATA_SLOT1    1
#define ZX_DATA_SLOT2    2
#define ZX_DATA_SLOT3    3

typedef struct
{
    int                slot;
    int                value[ZX_SLOT_SIZE];
}zx_bo_data_t;

typedef struct zx_dvfs_clamp_status
{
    union
    {
        unsigned int uint;
        struct
        {
            unsigned int gfx_dvfs_index_max  : 4;
            unsigned int gfx_dvfs_index_min  : 4;
            unsigned int s3vd_dvfs_index_max : 4;
            unsigned int s3vd_dvfs_index_min : 4;
            unsigned int vpp_dvfs_index_max  : 4;
            unsigned int vpp_dvfs_index_min  : 4;
            unsigned int reserved            : 8;
        };
    };
}zx_dvfs_clamp_status_t;

typedef struct zx_dvfs_force_index
{
    union
    {
        unsigned int uint;
        struct
        {
            unsigned int gfx_index      : 4; //later may be used by gfx
            unsigned int s3vd_index     : 4;
            unsigned int vpp_index      : 4;
            unsigned int engine_index   : 3;
            unsigned int reserved       : 17;
        };
    };
}zx_dvfs_force_index_t;

typedef struct zx_power_perf_mode_info
{
    int type;
    zx_dvfs_clamp_status_t value;
    zx_dvfs_force_index_t index;
}zx_power_perf_mode_info_t;

/*now only have perf mode, and only change dvfs clamp
slice switch will default use force two slice since power not saving muchand perf increase
*/

typedef enum
{
    ZX_POWER_MODE_PERF       = 0,   //force two slice, eclock can force
    ZX_POWER_MODE_SAVING     = 1,   //slice switch auto, eclock can force
    ZX_POWER_MODE_VIDEO      = 2,   //can use to force vcp/cpp 450M for 4k*2k video
    ZX_POWER_MODE_BALANCE    = 3,
    ZX_POWER_MODE_AGGRESSION = 4,
    ZX_POWER_MODE_NUM        = 5,
}zx_power_mode;

#define zx_align_down(val, align)  ((val) & ~((align) - 1))
#define zx_align_up(val, align)    (((val) + ((align) - 1)) & ~((align) - 1))

#endif /* __ZX_TYPES_H__*/

