/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "zx_driver.h"
#include "zxfb.h"

#define  ZX_FB_HOT_PLUG_ENABLE  0

#if 0
static struct fb_videomode zxfb_default_mode =  
/* 800x600 @ 60 Hz, 37.8kHz hsync */
{ 
    NULL, 60, 800, 600, 25000, 88, 40, 23, 1, 128, 4,
    FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
    FB_VMODE_NONINTERLACED
};
#endif

extern int zx_run_on_qt;

static struct fb_videomode zxfb_PANEL_mode =  
/* 800x600 @ 60 Hz, 37.8 kHz hsync */
{ 
    NULL, 60, 800, 600, 25000, 88, 40, 23, 1, 128, 4,
    FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT,
    FB_VMODE_NONINTERLACED 
};

static void *zxfb_lock_allocation(struct zxfb_dev *dev, unsigned int hAllocation,enum zx_mem_space mem_space, int tiled)
{
    zx_lock_allocation_t lock = {0};

    lock.device          = dev->hDevice;
    lock.allocation      = hAllocation;
    lock.acquire_apeture = tiled;

    krnl_lock_allocation(dev->adapter, &lock, mem_space);

    return lock.virt_addr;
}

static void zxfb_unlock_allocation(struct zxfb_dev *dev, unsigned int hAllocation,enum zx_mem_space mem_space)
{
    zx_unlock_allocation_t unlock = {0};

    unlock.device     = dev->hDevice;
    unlock.allocation = hAllocation;

    krnl_unlock_allocation(dev->adapter, &unlock, mem_space);
}

static void zxfb_destroy_modelist(struct list_head *head)
{
    struct list_head *curr, *next;

    list_for_each_safe(curr, next, head)
    {
        list_del(curr);
        kfree(curr);
    }
}

/*
static void dump_mode(struct fb_videomode *mode)
{
    zx_info("%d*%d@%d\n", mode->xres, mode->yres, mode->refresh);

    return;
}
static void dump_modelist(struct list_head *list)
{
    struct fb_modelist  *modelist = NULL;
    struct list_head *pos = NULL;

    list_for_each(pos, list) {

        modelist = list_entry(pos, struct fb_modelist, list);

        dump_mode(&modelist->mode);
    }

    zx_info("-----finish-----\n");
    return;
}
*/

static struct list_head* dup_modelist(struct list_head *list)
{
    struct list_head *pos;
    struct list_head *dup;
    struct fb_modelist  *modelist = NULL;

    dup = zx_calloc(sizeof(struct list_head));
    INIT_LIST_HEAD(dup);

    list_for_each(pos, list)
    {
        modelist = list_entry(pos, struct fb_modelist, list);

        fb_add_videomode(&modelist->mode, dup);
    }

    return dup;
}

static void zxfb_get_share_modelist(struct list_head *list1, struct list_head *list2, struct list_head *share_list)
{
    struct list_head *pos1=NULL;
    struct list_head *pos2=NULL;
    struct fb_modelist *modelist1=NULL;
    struct fb_modelist *modelist2=NULL;
    struct fb_videomode *m1=NULL;
    struct fb_videomode *m2=NULL;

    list_for_each(pos1, list1) 
    {
        modelist1 = list_entry(pos1, struct fb_modelist, list);
        m1 = &modelist1->mode;
        
        list_for_each(pos2, list2) 
        {
            modelist2 = list_entry(pos2, struct fb_modelist, list);
            m2 = &modelist2->mode;

            if ((m1->xres == m2->xres) && (m1->yres == m2->yres))
            {
                fb_add_videomode(m1, share_list);
            }
       }
    }
}

static void zxfb_outputs_share_modelist(struct zxfb_outputs_config *outputs_config)
{
    struct zxfb_output *head_output = NULL;
    struct list_head *tmp_list = NULL;
    struct list_head *share_list = NULL;
    int i = 0;
    int j = 0;

    for(i = 0; i < outputs_config->num_output; i++)
    {
        head_output = &outputs_config->outputs[i];
        if(head_output->connected && (head_output->crtc != -1))
        {
            break;
        }
    }

    if(i == outputs_config->num_output)
    {
        return;
    }

    tmp_list = dup_modelist(&head_output->modelist);

    for(j = i+1; j < outputs_config->num_output; j++)
    {
        struct zxfb_output *next_output = &outputs_config->outputs[j];

        if(!next_output->connected || (next_output->crtc == -1))
        {
            continue;
        }

        share_list = zx_calloc(sizeof(struct list_head));
        INIT_LIST_HEAD(share_list);
        
        zxfb_get_share_modelist(tmp_list, &next_output->modelist, share_list);

        if(!list_empty(share_list))
        {
            fb_destroy_modelist(tmp_list);
            zx_free(tmp_list);
            tmp_list = share_list;
        }
        else
        {
            zx_free(share_list);
            next_output->crtc = -1; // do not have same mode, ignore next output
        }
    }

    outputs_config->shared_modelist = tmp_list;
}

static void zxfb_combine_output_crtc(struct zxfb_dev *dev, struct zxfb_outputs_config *outputs_config)
{
    struct zxfb_output *output=NULL;
    zx_dev_comb_t dev_comb;
    int i = 0, j = 0;

    zx_memset(&dev_comb, 0, sizeof(zx_dev_comb_t));

    for(i = 0; i < outputs_config->num_output; i++)
    {
        output = &outputs_config->outputs[i];
        if(output->connected)
        {
            dev_comb.devices  |= output->type;
            if(++j >= outputs_config->num_crtc)
            {
                break;
            }
        }
    }

    krnl_get_device_comb(dev->adapter, &dev_comb);

    for(i = 0; i < outputs_config->num_output; i++)
    {
        output = &outputs_config->outputs[i];
        for(j = 0; j < sizeof(dev_comb.iga_dev)/sizeof(dev_comb.iga_dev[0]); j++)
        {
            if(output->type == dev_comb.iga_dev[j])
            {
                output->crtc = j;
            }
        }
    }

    if (zx_run_on_qt)
    {
        // force to CRTC0
        zx_info("num_output = %d\n", outputs_config->num_output);

        for(i = 0; i < outputs_config->num_output; i++)
        {
            output = &outputs_config->outputs[i];

            output->crtc = 0;
        }
    }

    return;
}

static int zxfb_merge_devcice_mode_list(const struct list_head* src_list, void* modes, int mode_num, struct list_head* dst_list)
{
    struct list_head *pos;
    struct fb_modelist *modelist;
    struct fb_videomode *video_mode;
    zx_mode_t*  device_mode = (zx_mode_t*)modes;
    int  status = -1, i = 0, found = 0;

    list_for_each(pos, src_list) 
    {
        modelist = list_entry(pos, struct fb_modelist, list);
        video_mode = &modelist->mode;
        if(!device_mode || !mode_num)
        {
            fb_add_videomode(video_mode, dst_list);
            status = 0;
        }
        else
        {
            found = 0;
            for(i = 0; i < mode_num; i++)
            {
                if((video_mode->xres == device_mode[i].xres) && (video_mode->yres == device_mode[i].yres))
                {
                    unsigned int src_refresh = video_mode->refresh * 100;
                    unsigned int dst_refresh = device_mode[i].refresh_rate;
                    unsigned int delta = (src_refresh >= dst_refresh)? (src_refresh - dst_refresh) : (dst_refresh - src_refresh);
                    if(delta <= 100)
                    {
                        found = 1;
                        video_mode->refresh = device_mode[i].refresh_rate/100;
                        break;
                    }
                }
            }
            if(found)
            {
                fb_add_videomode(video_mode, dst_list);
                status = 0;
            }
        }
    }
    return  status;
}

static void zxfb_devicemode_to_videomode(struct zxfb_dev *dev, struct fb_videomode *video_mode, zx_mode_t* device_mode, unsigned int type)
{
    zx_mode_timing_info_t timing_info = {0};

    timing_info.uDev = type;
    timing_info.xRes = device_mode->xres;
    timing_info.yRes = device_mode->yres;
    timing_info.refresh = device_mode->refresh_rate;
    timing_info.bInterlaced = (device_mode->interlace_progressive_caps == 0x1) ? 0 : 1;
    timing_info.aspectRatio = device_mode->aspect_ratio_caps;

    krnl_get_mode_timing(dev->adapter, &timing_info);

    video_mode->name = NULL;
    video_mode->refresh = timing_info.refresh /100;
    video_mode->xres = timing_info.xRes;
    video_mode->yres = timing_info.yRes;
    video_mode->pixclock = timing_info.Clock;
    video_mode->left_margin = timing_info.HSyncStart - timing_info.HDisplay;
    video_mode->right_margin = timing_info.HTotal - timing_info.HSyncEnd;
    video_mode->upper_margin = timing_info.VSyncStart - timing_info.VDisplay;
    video_mode->lower_margin = timing_info.VTotal - timing_info.VSyncEnd;
    video_mode->hsync_len = timing_info.HSyncEnd - timing_info.HSyncStart;
    video_mode->vsync_len = timing_info.VSyncEnd - timing_info.VSyncStart;
    video_mode->sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_VERT_HIGH_ACT;
    video_mode->vmode = timing_info.bInterlaced ? FB_VMODE_INTERLACED : FB_VMODE_NONINTERLACED;
    
    if(device_mode->isDetailedTiming)
    {
        video_mode->flag |= FB_MODE_IS_DETAILED;
    }
    if(device_mode->isStandardTiming)
    {
        video_mode->flag |= FB_MODE_IS_STANDARD;
    }
}

static int zxfb_probe_output(struct zxfb_dev *dev, struct zxfb_output *output)
{
    zx_output_caps_t output_caps = {0};
    unsigned char ext_edid = 0;
    struct list_head  temp_modelist;
    int  status = 0;
    
    output_caps.type = output->type;

    krnl_query_output_caps(dev->adapter, &output_caps, FALSE);

    if(!output_caps.connected)
    {
        zx_info("warning, probe a un-connected output: 0x%x.\n", output->type);
        
        return -1;
    }

    output->connected = TRUE;
    output->num_mode  = output_caps.mode_num;
    if(output->num_mode)
    {
        output_caps.modes = zx_calloc(output->num_mode * sizeof(zx_mode_t));
    }

    if(output_caps.edid_size)
    {
        output_caps.edid  = zx_calloc(output_caps.edid_size);
    }

    krnl_query_output_caps(dev->adapter, &output_caps, FALSE);

    output->edid = output_caps.edid;
    output->modes = output_caps.modes;

    if(output->edid != NULL)
    {
        fb_edid_to_monspecs(output->edid, &output->monspecs);
        ext_edid = *((unsigned char*)output->edid + 126);
        if(ext_edid > 3)
        {
            ext_edid = 3;
        }

        if(ext_edid != 0)
        {
            int i = 0;
            unsigned char *edid = NULL;

            /* parse other blocks */
            for(i = 0;i < ext_edid;i++)
            {
                edid = (unsigned char*)output->edid + (i + 1)*128;
                fb_edid_add_monspecs(edid, &output->monspecs);
            }
        }
    }

    INIT_LIST_HEAD(&output->modelist);

    if(output->monspecs.modedb != NULL)
    {
        zx_info("use device mode and modedb to gen mode list\n");
        INIT_LIST_HEAD(&temp_modelist);
        fb_videomode_to_modelist(output->monspecs.modedb, output->monspecs.modedb_len, &temp_modelist);
        status = zxfb_merge_devcice_mode_list(&temp_modelist, output->modes, output->num_mode, &output->modelist);
        fb_destroy_modelist(&temp_modelist);

        //patch for panel
        //maybe there is no share mode between panel and other device, fake a small mode to prevent panel cannot be light
        if((output_caps.monitor_type == UT_OUTPUT_TYPE_PANEL) && (dev->chip_id == CHIP_CHX002))
        {
            fb_add_videomode(&zxfb_PANEL_mode, &output->modelist);
        }
    }

    //fb cannot get mode,use cbios mode, CONFIG_FB_MODE_HEKPERS not set in NeoKylin 
    if(list_empty(&output->modelist))
    {
        int i = 0;
        zx_mode_t* device_mode = (zx_mode_t*)output->modes;

        for(i = 0; i < output->num_mode; i++)
        {
            struct fb_videomode video_mode = {0};
            zxfb_devicemode_to_videomode(dev, &video_mode, &device_mode[i], output->type);
            fb_add_videomode(&video_mode, &output->modelist);
        }
    }

    return 0;
}

static void zxfb_remove_output(struct zxfb_dev *dev, struct zxfb_output *output)
{
    fb_destroy_modedb(output->monspecs.modedb);

    zxfb_destroy_modelist(&output->modelist);

    zx_free(output->edid);
    zx_free(output->modes);

    zx_memset(&output->monspecs, 0, sizeof(output->monspecs));
    zx_memset(&output->curr_mode, 0, sizeof(output->curr_mode));
    output->num_mode  = 0;
    output->edid      = NULL;
    output->modes = NULL;
    output->connected = FALSE;
    output->crtc = -1;
};

static int zxfb_init_outputs_config(struct zxfb_dev *dev, struct zxfb_outputs_config *outputs_config)
{
    struct zxfb_output *output   = NULL;
    zx_display_caps_t display_caps = {0};
    unsigned int support_outputs  = 0;
    int  support_outputs_num = 0;
    int    i = 0, j = 0;

    krnl_query_display_caps(dev->adapter, &display_caps);

    outputs_config->support_outputs   = display_caps.outputs_supported;
    outputs_config->connected_outputs = display_caps.outputs_connected;
    outputs_config->num_crtc          = display_caps.num_crtcs;
    outputs_config->num_output        = display_caps.num_outputs;

    support_outputs = outputs_config->support_outputs;
    while(support_outputs)
    {
        if(support_outputs & (1<<i))
        {
            support_outputs_num ++;
            support_outputs  &= ~(1<<i);
        }
        i++;
    }
    outputs_config->outputs = zx_calloc(sizeof(struct zxfb_output) * support_outputs_num);

    if(outputs_config->connected_outputs == 0)
    {
        zx_error("zxfb_init_outputs_config: no device connected! \n");
        return -1;
    }

    i = 0;
    support_outputs = outputs_config->support_outputs;

    while(support_outputs)
    {
        if(support_outputs & (1 << i))
        {
            output = &outputs_config->outputs[j++];
            output->type = (1 << i);
            output->crtc = -1;
            if(outputs_config->connected_outputs & output->type)
            {
                zxfb_probe_output(dev, output);
            }

            support_outputs &= (~output->type);
        }

        i++;
    }

    zxfb_combine_output_crtc(dev, outputs_config);

    zxfb_outputs_share_modelist(outputs_config);

    return 0;
}

static void zxfb_destroy_outputs_config(struct zxfb_dev *dev, struct zxfb_outputs_config *outputs_config)
{
    struct zxfb_output *output = NULL;
    int    i;

    for(i = 0; i < outputs_config->num_output; i++)
    {
        output = &outputs_config->outputs[i];

        if(output->connected)
        {
            zxfb_remove_output(dev, output);
        }
    }

    fb_destroy_modelist(outputs_config->shared_modelist);
}

#if  ZX_FB_HOT_PLUG_ENABLE
static int zxfb_update_outputs_config(struct zxfb_dev *dev, struct zxfb_outputs_config *outputs_config)
{
    struct zxfb_output *output   = NULL;

    zx_display_caps_t display_caps = {0};

    zx_dpms_set_t          dpms     = {0};

    int    i = 0;

    krnl_query_display_caps(dev->adapter, &display_caps);

    outputs_config->connected_outputs = display_caps.outputs_connected;

    for(i = 0; i < outputs_config->num_output; i++)
    {
        output = &outputs_config->outputs[i];
        //turn off all active device
        if(output->crtc != -1)
        {
            dpms.output = output->type;
            dpms.mode   = 4;
            krnl_set_dpms(dev->adapter, &dpms);
        }

        if(output->connected)
        {
            zxfb_remove_output(dev, output);
        }

        if(output->type & outputs_config->connected_outputs)
        {
            zxfb_probe_output(dev, output);
        }
    }

    zxfb_combine_output_crtc(dev, outputs_config);

    zxfb_outputs_share_modelist(outputs_config);

    return 0;
}
#endif

static int zxfb_create_device(struct zxfb_dev *dev)
{
    zx_create_device_t    create_device = {0};
    zx_create_context_t   create_context = {0};
    int                    status  = 0;

    //create client here
    krnl_graphics_client_create(dev->adapter, NULL, ZX_CLIENT_TYPE_FB);
    krnl_enter_zx_graphics_mode(dev->adapter, NULL);

    status = krnl_create_device(dev->adapter, NULL, &create_device.device);
    if(status == 0)
    {
        dev->hDevice = create_device.device;

        create_context.device = dev->hDevice;
        create_context.engine_index = 0;
        status = krnl_create_context(dev->adapter, &create_context, ZX_MEM_KERNEL);
        if(status == 0)
        {
            dev->hContext = create_context.context;
        }
        else
        {
            zx_info("krnl_create_context failed. errno: %d \n", status);
            if(dev->hDevice)
            {
                krnl_destroy_device(dev->adapter, dev->hDevice);
            }
        }
    }
    else
    {
        zx_info("krnl_create_device failed. errno: %d \n", status);
    }

    return status;
}

static int zxfb_destroy_device(struct zxfb_dev *dev)
{
    zx_destroy_context_t destroy_context = {0};
    int                   status  = 0;

    if(dev->hDevice && dev->hContext)
    {
        destroy_context.device  = dev->hDevice;
        destroy_context.context = dev->hContext;
        status = krnl_destroy_context(dev->adapter, &destroy_context);
    }

    if(dev->hDevice)
    {
        krnl_destroy_device(dev->adapter, dev->hDevice);
    }
    
    dev->hDevice  = 0;
    dev->hContext = 0;

    return status;

}


static int zxfb_init_dev(struct zxfb_dev *dev)
{
    zx_query_info_t query      = {0};
    query.type = ZX_QUERY_CHIP_ID;
    
    krnl_query_info(dev->adapter, &query);
    dev->chip_id = query.value;

    /* init chip private info */
    switch (query.value)
    {
    case CHIP_ELT:
    case CHIP_ELT1K:
        dev->local_dynamic  = TRUE;
        break;

    case CHIP_E2UMA:
    case CHIP_CHX001:
    case CHIP_CHX002:        
    case CHIP_ELT2K:
    case CHIP_ZX2000:
        dev->local_dynamic  = FALSE;
        break;
    default:
        break;
    }

    /* Create device and context */
    zxfb_create_device(dev);

    zxfb_init_outputs_config(dev, &dev->outputs_config);

    return 0;
}

static const struct fb_videomode* zxfb_get_best_mode(struct zxfb_dev *dev)
{
    struct zxfb_outputs_config *outputs_config = &dev->outputs_config;
    struct zxfb_output *output = NULL;
    const struct fb_videomode *best_mode = NULL;
    int i = 0, j = 0;
    zx_mode_t* device_mode  = NULL;
    
    //1. share list is not NULL, find mode in it
    if(! list_empty(outputs_config->shared_modelist))
    {
        struct fb_videomode mode = {0};

        //1.1 get configured mode
        if(dev->mode_option != NULL)
        {
            zx_sscanf(dev->mode_option, "%dx%d-%d@%d", &mode.xres, &mode.yres, &dev->default_bpp, &mode.refresh);
            best_mode = fb_find_nearest_mode(&mode, outputs_config->shared_modelist);
        }
        //1.2 get biggest preffered mode
        if(best_mode == NULL)
        {
            struct fb_modelist  *modelist = NULL;
            struct list_head *pos = NULL;

            list_for_each(pos, outputs_config->shared_modelist) {
                modelist = list_entry(pos, struct fb_modelist, list);

                if((modelist->mode.flag & FB_MODE_IS_FIRST) && (modelist->mode.flag & FB_MODE_IS_DETAILED))
                {
                    if((best_mode == NULL) || (modelist->mode.xres * modelist->mode.yres > best_mode->xres * best_mode->yres))
                    {
                        best_mode = &modelist->mode;
                    }
                }
            }
        }
        //1.3 get biggest mode
        if(best_mode == NULL)
        {
            mode.xres = 10000;
            mode.yres = 10000;
            mode.refresh = 1000;

            best_mode = fb_find_nearest_mode(&mode, outputs_config->shared_modelist);
        }

        for(i = 0; i < outputs_config->num_output; i++)
        {
            output = &outputs_config->outputs[i];
            if(!output->connected || (output->crtc == -1))
            {
                continue;
            }


            device_mode = (zx_mode_t*)output->modes;

            for (j = 0; j < output->num_mode; j++)
            {   
                if ((device_mode[j].xres == best_mode->xres) && (device_mode[j].yres == best_mode->yres))
                {
                    output->curr_mode.xres = best_mode->xres;
                    output->curr_mode.yres = best_mode->yres;
                    output->curr_mode.refresh = device_mode[j].refresh_rate / 100;

                    zx_info("select output 0x%x xres %d yres %d rr %d\n", output->type, output->curr_mode.xres, output->curr_mode.yres, output->curr_mode.refresh);
                    break;
                }
            }
        }
    }
    //2. share list is empty
    else
    {
        zx_assert(FALSE);
    }

    return best_mode;
}

static int zxfb_init_var(struct fb_info *info, struct zxfb_dev *dev)
{
    const struct fb_videomode *best_mode = NULL;
    int status = 0;

    best_mode = zxfb_get_best_mode(dev);    

    if(best_mode)
    {
        INIT_LIST_HEAD(&info->modelist);

        fb_add_videomode(best_mode, &info->modelist);

        fb_videomode_to_var(&info->var, best_mode);

        info->var.bits_per_pixel = dev->default_bpp;

        status = zxfb_ops.fb_check_var(&info->var, info);
    }
    else
    {
        zx_info("can not found a suitable mode.\n");
        status = -1;
    }

    return status;
}

int zxfb_create_screen(struct zxfb_dev *dev, struct zxfb_screen *screen)
{
    zx_2d_screen_t create = {0};
    create.width  = screen->width;
    create.height = screen->height;
    create.bpp    = screen->bpp;
    create.device = dev->hDevice;
    zx_init_2d_screen(dev->zx, NULL, &create);
    
    screen->pitch       = create.pitch;
    screen->gbd         = create.gbd;
    screen->hAllocation = create.hAllocation;
    screen->fbOffset    = create.fbOffset;
    screen->fbVirtual   = zxfb_lock_allocation(dev, create.hAllocation, 1, create.tiled);
    return 0;
}

int zxfb_destroy_screen(struct zxfb_dev *dev, struct zxfb_screen *screen)
{
    if(screen->fbVirtual)
    {
        zxfb_unlock_allocation(dev, screen->hAllocation, 1);
        screen->fbVirtual = NULL;
    }

    if(screen->hAllocation)
    {
        zx_2d_screen_t create = {0};

        zx_deinit_2d_screen(dev->zx, &create);

        screen->hAllocation = 0;
    }

    return 0;
}

int zxfb_handle_event(zx_card_t *zx, unsigned int events)
{
#if ZX_FB_HOT_PLUG_ENABLE 

    struct fb_info      *info   = zx->info;
    struct zxfb_dev    *dev    = info->par;
    struct zxfb_outputs_config *outputs_config = &dev->outputs_config;
    const struct fb_videomode *best_mode = NULL;

    if(events & ZX_EVENT_HOTPLUG)
    {
        //zx_info("%s() HOTPLUG detected\n", __func__);

        zxfb_update_outputs_config(dev, outputs_config);

        best_mode = zxfb_get_best_mode(dev);       

        if(best_mode != NULL)
        {
            struct fb_event event = {0};

            event.info = info;

            acquire_console_sem();

            zxfb_destroy_modelist(&info->modelist);

            fb_add_videomode(best_mode, &info->modelist);

            fb_notifier_call_chain(FB_EVENT_NEW_MODELIST, &event);

            release_console_sem();
        }
        else
        {
            zx_info("%s() find no mode!\n", __func__);
        }

    }
#endif

    return 0;
}

static int zxfb_mmap(struct fb_info *info, struct vm_area_struct *vma)
{
    unsigned long phys_addr;
    unsigned long size;

    int ret = -EINVAL;

    struct zxfb_dev *dev = info->par;
    
    zx_map_argu_t map;

    if(vma->vm_end - vma->vm_start == 0)
    {
        return 0;
    }

    size = vma->vm_end - vma->vm_start;

    if(vma->vm_pgoff > (~0UL >> PAGE_SHIFT))
    {
        return -EINVAL;
    }

    phys_addr = dev->screen.fbOffset;

    krnl_get_map_allocation_info(dev->adapter, dev->screen.hAllocation, &map);

    zx_assert(map.size >= size);

    map.flags.cache_type = ZX_MEM_WRITE_COMBINED;
    map.flags.mem_space  = ZX_MEM_USER;
    
    switch(map.flags.mem_type)
    {
      case ZX_SYSTEM_RAM:
        ret = zx_map_system_ram(vma, &map);
        break;
      case ZX_SYSTEM_IO:
        ret = zx_map_system_io(vma, &map);
        break;
      default:
        ret = -EINVAL;
        zx_error("%s, unknown memory map type", __func__);
    }
    
    return ret;
}

int zxfb_probe(zx_card_t *zx, char *mode_string, int flip)
{
    struct fb_info   *info;
    struct zxfb_dev *dev;
    int status = 0;
    bus_config_t bus_config = {0};
    unsigned int mmioBase = 0;
    unsigned int mmioSize = 0;

    //aquire console lock before access frame buffer.
    zx_console_lock();
    
    zx_get_bus_config(zx->pdev, &bus_config);
    mmioBase = bus_config.reg_start_addr[0];
    mmioSize = bus_config.reg_end_addr[0] - bus_config.reg_start_addr[0] + 1;

    /* create fb_info */
    info = framebuffer_alloc(sizeof(struct zxfb_dev), &zx->pdev->dev);

    dev = info->par;
    dev->zx            = zx;
    dev->adapter        = zx->adapter;
    dev->mode_option    = mode_string;
    dev->flip           = flip;
    dev->default_width  = 1920;
    dev->default_height = 1080;
    dev->default_bpp    = 32;
    dev->screen_resize  = TRUE;
    dev->yoffset        = -1;
    dev->stream_type    = ZX_STREAM_PS;

    dev->fb_start = bus_config.mem_start_addr[0];

    zxfb_init_dev(dev);

    zxfb_init_var(info, dev);

    if(dev->local_dynamic)
    {
        zxfb_ops.fb_mmap = zxfb_mmap;
    }

    info->screen_base    = dev->screen.fbVirtual;
    info->fbops          = &zxfb_ops;
    info->screen_size    = dev->screen.pitch * info->var.yres;
    info->pseudo_palette = dev->pseudo_palette;

    info->flags = FBINFO_DEFAULT | (FBINFO_HWACCEL_COPYAREA | FBINFO_HWACCEL_FILLRECT | FBINFO_HWACCEL_IMAGEBLIT);

    info->fix.type        = FB_TYPE_PACKED_PIXELS;
    info->fix.visual      = FB_VISUAL_TRUECOLOR;
    info->fix.type_aux    = 0;
    info->fix.xpanstep    = 1;
    info->fix.ypanstep    = 1;
    info->fix.ywrapstep   = 0;
    info->fix.accel       = 0;
    info->fix.line_length = dev->screen.pitch;
    info->fix.smem_start  = dev->screen.fbOffset + dev->fb_start;
    info->fix.smem_len    = dev->screen.pitch * info->var.yres;
    info->fix.mmio_start  = mmioBase;
    info->fix.mmio_len    = mmioSize;
    info->var.activate    = FB_ACTIVATE_NOW;

    zx_vsprintf(info->fix.id, "%s", "zxfb");

    //Fix black window via xf86VTSwitch when suspend/resume
    info->skip_vt_switch  = 1;

    zx->info = info;

    //release console lock before register framebuffer to prevent deadlock.
    zx_console_unlock();
    zx_msleep(500);

    
#ifndef ZX_HW_NULL
    status = register_framebuffer(info);
#endif

    return status;
}

int zxfb_remove(zx_card_t *zx)
{
    struct fb_info   *info = zx->info;
    struct zxfb_dev *dev  = info->par;

    krnl_power_off_crtc(dev->adapter);

#ifndef ZX_HW_NULL
    unregister_framebuffer(info);
#endif

    zxfb_destroy_screen(dev, &dev->screen);

    zxfb_destroy_outputs_config(dev, &dev->outputs_config);

    zxfb_destroy_stream(info);
    zxfb_destroy_device(dev);

    dev->adapter = NULL;
    dev->zx     = NULL;

    framebuffer_release(info);

    zx->info = NULL;

    return 0;
}

int zxfb_shutdown(zx_card_t *zx)
{
    struct fb_info   *info = zx->info;
    struct zxfb_dev *dev  = info->par;

    dev->disabled = 1;

    return 0;
}

void zxfb_set_pm(zx_card_t *zx, int bsuspend)
{
    struct fb_info *info = zx->info;

    int state = bsuspend ? 1 : 0; 

    if(info == NULL)
    {
        return;
    }

    acquire_console_sem();

    fb_set_suspend(info, state);

    release_console_sem();
}

#define VESA_MEM_ALIGN      64*1024
void zxfb_detect_vesa(zx_card_t *zx)
{
    struct fb_info   *vesafb    = NULL;
    bus_config_t     bus_config = {0};

    int i, vesafb_found = 0, vga16fb_found = 0;

    zx_get_bus_config(zx->pdev, &bus_config);

    for(i = 0; i < num_registered_fb; i++)
    {
        vesafb = registered_fb[i];

        if((vesafb != NULL) && 
           (vesafb->fix.smem_start >= bus_config.mem_start_addr[0]) && 
           (vesafb->fix.smem_start <  bus_config.mem_end_addr[0]))
        {
            vesafb_found = 1;
            break;
        } 

        if((vesafb != NULL) && 
           (vesafb->fix.smem_start == 0xA0000))
        {
            vga16fb_found = 1;
            break;
        }
    }

    if(zx->zxfb_enable && (vga16fb_found || vesafb_found))
    {
        zx_info("%s detected, conflicted with zxfb, remove %s.\n", vesafb->fix.id, vesafb->fix.id);

        unregister_framebuffer(vesafb);
        release_mem_region(vesafb->fix.smem_start, screen_info.lfb_size * 65536);

        vga16fb_found = 0;
        vesafb_found  = 0;
    }

    if(vga16fb_found)
    {
         zx->vga16fb_enable = 1;
         zx->reserved_vmem  = 256 * 1024;

         zx_info("vga16fb detected, reserved %dK vmem\n", zx->reserved_vmem/1024);
    }

    if(vesafb_found)
    {
        zx->vesafb_mode.crtc       = 0;
        zx->vesafb_mode.output     = 0;
        zx->vesafb_mode.bpp        = vesafb->var.bits_per_pixel;
        zx->vesafb_mode.width      = vesafb->var.xres;
        zx->vesafb_mode.height     = vesafb->var.yres;
        zx->vesafb_mode.dst_width   = vesafb->var.xres;
        zx->vesafb_mode.dst_height  = vesafb->var.yres;
        zx->vesafb_mode.dst_refresh = 0;
        zx->vesafb_mode.scaler_type = 0;
        zx->vesafb_mode.interlace = 0;
        zx->vesafb_mode.scale_width = vesafb->var.xres;
        zx->vesafb_mode.scale_height = vesafb->var.yres;

        zx->vesafb_enable = 1;
        zx->reserved_vmem = (vesafb->fix.smem_len + VESA_MEM_ALIGN - 1) & (~(VESA_MEM_ALIGN-1));

        zx_info("vesafb detected, %dx%d@%d, reserved %dK video memory for it\n", 
             vesafb->var.xres, vesafb->var.yres, vesafb->var.bits_per_pixel,zx->reserved_vmem/1024);
    }
}

