/* 
* Copyright 1998-2014 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2014 S3 Graphics, Inc. All Rights Reserved.
* Copyright 2013-2014 Shanghai Zhaoxin Semiconductor Co., Ltd. All Rights Reserved.
*
* This file is part of zx.ko
* 
* zx.ko is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* zx.ko is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "zxfb.h"
#include "zx_ioctl.h"

static int zxfb_open(struct fb_info* info, int user)
{
    int ret = 0;

    return ret;
}

static int zxfb_release(struct fb_info* info, int user)
{
    int ret = 0;

    return ret;
}

static int zxfb_check_var(struct fb_var_screeninfo* var, struct fb_info* info)
{
    struct zxfb_dev *dev = info->par;
    int height = 0;
    int status = 0;

    switch (var->bits_per_pixel)
    {
    case 16:
        var->red.offset   = 11;
        var->green.offset = 5;
        var->blue.offset  = 0;
        var->red.length   = 5;
        var->green.length = 6;
        var->blue.length  = 5;
        break;
    case 32:
        var->red.offset   = 16;
        var->green.offset = 8;
        var->blue.offset  = 0;
        var->red.length   = 8;
        var->green.length = 8;
        var->blue.length  = 8;
        break;

    default:
        status = -EINVAL;
        break;
    }

    /* create new screen buffer */
    if(dev->screen.hAllocation == 0)
    {
        if(dev->screen_resize)
        {
            dev->screen.width  = info->var.xres;
            height = info->var.yres;
            dev->screen.height = dev->flip ? height * 2 : height;
            dev->screen.bpp    = info->var.bits_per_pixel;
        }
        else 
        {
             if((var->xres > dev->default_width) || 
                (var->xres > dev->default_height))
             {
                 var->xres = dev->default_width;
                 var->yres = dev->default_height;

                 zx_info("required screen size(%d x %d) exceed real screen size (%d x %d), punt to real screen size", 
                     var->xres, var->yres, dev->default_width, dev->default_height);
             }

            dev->screen.width  = dev->default_width;
            dev->screen.height = dev->flip ? dev->default_height * 2 : dev->default_height;
            dev->screen.bpp    = info->var.bits_per_pixel;           
        }

        zxfb_create_screen(dev, &dev->screen);
    }
    else
    {
        if(dev->screen_resize)
        {
            if(var->xres != dev->screen.width ||
               var->yres != dev->screen.height ||
               var->bits_per_pixel != dev->screen.bpp)
            {
                zxfb_destroy_screen(dev, &dev->screen);

                dev->screen.width  = info->var.xres;
                height = info->var.yres;
                dev->screen.height = dev->flip ? height * 2 : height;
                dev->screen.bpp    = info->var.bits_per_pixel;

                zxfb_create_screen(dev, &dev->screen);
            }
        }
        else
        {
             if((var->xres > dev->screen.width) || 
                (var->xres > dev->screen.height))
             {
                 var->xres = dev->screen.width;
                 var->yres = dev->screen.height;

                 zx_info("required screen size(%d x %d) exceed real screen size (%d x %d), punt to real screen size", 
                     var->xres, var->yres, dev->screen.width, dev->screen.height);
             }

        }
    }

    info->screen_base     = dev->screen.fbVirtual;
    info->screen_size     = dev->screen.pitch * info->var.yres;
    info->fix.line_length = dev->screen.pitch;
    info->fix.smem_start  = dev->screen.fbOffset + dev->fb_start;
    info->fix.smem_len    = dev->screen.pitch * info->var.yres;

    zx_info("var:(%dx%d-%d), zxfb screen: (%dx%d-%d).\n",
         var->xres, var->yres, var->bits_per_pixel, 
         dev->screen.width, dev->screen.height, dev->screen.bpp);
    
    return status;
}

int zxfb_set_stream(struct fb_info* info, struct zxfb_output *output)
{
    struct zxfb_dev *dev = info->par;
    zx_ps_info_t     set_ps = {0};
    int status = 0;
    zx_stream_info_t *stream_info  = NULL;

    if(dev->chip_id == CHIP_ELT2K || 
       dev->chip_id == CHIP_E2UMA ||
       dev->chip_id == CHIP_CHX001 ||
       dev->chip_id == CHIP_CHX002)
    {
        zx_display_frame_t *display_frame = NULL;
        zx_create_stream_t create_stream = {0};
        ZX_STREAM_TYPE     stream_type = dev->stream_type;
        int                 crtc_index = output->crtc;

        display_frame = &dev->display_frame[crtc_index];
        display_frame->hContext = dev->hContext;
        display_frame->crtc              = crtc_index;
        display_frame->hDevice           = dev->hDevice;
        display_frame->retire_fence_fd   = -1;
        display_frame->need_retire_fence = 0;
        display_frame->active_stream_mask   = 1 << stream_type;
        display_frame->acquired_stream_mask = 1 << stream_type;
        
        stream_info  = &(display_frame->stream_info[stream_type]);
        if (stream_info->hStream == 0)
        {
            create_stream.hDevice     = display_frame->hDevice;
            create_stream.crtc        = display_frame->crtc;
            create_stream.stream_type = stream_type;
            create_stream.timeout     = 2000;

            status =  krnl_create_stream(dev->adapter, &create_stream);
            if(status)
            {
                zx_info("krnl_create_stream failed. errno: %d \n", status);
                return -1;
            }

            stream_info->hStream = create_stream.hStream;
        }

        stream_info->surface = dev->screen.hAllocation;
        stream_info->stream_op_mask |= STREAM_MASK_UPDATE;
        stream_info->src_rect.x1 = 0;
        stream_info->src_rect.y1 = 0;
        stream_info->src_rect.x2 = dev->screen.width;
        stream_info->src_rect.y2 = dev->screen.height;

        stream_info->dst_rect.x1 = 0;
        stream_info->dst_rect.y1 = 0;
        stream_info->dst_rect.x2 = dev->screen.width;
        stream_info->dst_rect.y2 = dev->screen.height;

        zx_info("%s(): iga-%d, stream_type-%d, src[%d,%d, %d,%d], dst[%d,%d, %d,%d]\n", __func__, crtc_index, stream_type,
            stream_info->src_rect.x1, stream_info->src_rect.y1, stream_info->src_rect.x2, stream_info->src_rect.y2,
            stream_info->dst_rect.x1, stream_info->dst_rect.y1, stream_info->dst_rect.x2, stream_info->dst_rect.y2);

        status = krnl_display_frame(dev->adapter, display_frame);
    }
    else
    {
        set_ps.crtc          = output->crtc;;
        set_ps.surface = dev->screen.hAllocation;
        krnl_set_ps(dev->adapter, &set_ps);
    }
    
    return status;
}

void  zxfb_destroy_stream(struct fb_info* info)
{
    struct zxfb_dev *dev = info->par;
    struct zxfb_outputs_config *outputs_config = &dev->outputs_config;
    zx_display_frame_t *display_frame = NULL;
    zx_stream_info_t   *stream_info  = NULL;
    zx_destroy_stream_t destroy_stream = {0};
    
    destroy_stream.hDevice = dev->hDevice;

    if(dev->chip_id == CHIP_ELT2K ||
       dev->chip_id == CHIP_E2UMA ||
       dev->chip_id == CHIP_CHX001 ||
       dev->chip_id == CHIP_CHX002)
    {
        ZX_STREAM_TYPE stream_type = dev->stream_type;
        struct zxfb_output *output = NULL;
        int i = 0;

        for (i = 0; i < outputs_config->num_output; i++)
        {

            output = &outputs_config->outputs[i];
            if(!output->connected)
            {
                continue;
            }

            display_frame = &dev->display_frame[output->crtc];
            stream_info  = &(display_frame->stream_info[stream_type]);
            destroy_stream.hStream = stream_info->hStream;
            
            krnl_destroy_stream(dev->adapter, &destroy_stream);
            
            stream_info->hStream = 0;
        }
    }
}

/* set the video mode according to info->var */
static int zxfb_set_par(struct fb_info* info)
{
    struct zxfb_dev *dev = info->par;
    struct fb_var_screeninfo  *var  = &info->var;
    struct zxfb_outputs_config *outputs_config = &dev->outputs_config;
    struct zxfb_output         *output = NULL;

    zx_mode_info_t         mode_set = {0};
    zx_dpms_set_t          dpms = {0};
    int i = 0;

    for(i = 0; i < outputs_config->num_output; i++)
    {
        output = &outputs_config->outputs[i];
        if(!output->connected || (output->crtc == -1))
        {
            continue;
        }

        mode_set.crtc   = output->crtc;
        mode_set.output = output->type;
        mode_set.bpp    = var->bits_per_pixel;

        mode_set.width        = 
        mode_set.scale_width  = output->curr_mode.xres;
        mode_set.height       = 
        mode_set.scale_height = output->curr_mode.yres;

        mode_set.dst_width    = output->curr_mode.xres;
        mode_set.dst_height   = output->curr_mode.yres;

        mode_set.dst_refresh  = output->curr_mode.refresh * 100;

        krnl_set_mode(dev->adapter, &mode_set);

        dpms.output = output->type;
        dpms.mode   = 0;
        krnl_set_dpms(dev->adapter, &dpms);

        zxfb_set_stream(info, output); 
    }

    return 0;
}


/* set color register */
static int zxfb_setcolreg(unsigned regno, unsigned red, unsigned green, unsigned  blue, unsigned transp, struct fb_info* info)
{
    struct zxfb_dev *dev = info->par;   

    if (regno >= 16)
    {
        return -1;
    }

    switch (info->var.bits_per_pixel)
    {
    case 16:
        dev->pseudo_palette[regno] = (red & 0xf800) | ((green & 0xfc00) >> 5) | ((blue & 0xf800) >> 11);
        break;

    case 32:
        transp >>= 8;
        red >>= 8;
        green >>= 8;
        blue >>= 8;
        dev->pseudo_palette[regno] = (transp << 24) | (red << 16 | (green << 8) | blue);
        break;
    }

    return 0;
}

/* blank display */
static int zxfb_blank(int blank_mode, struct fb_info* info)
{
    int ret = 0;
    struct zxfb_dev *dev = info->par;
    zx_dpms_set_t dpms = {0};
    int i;
    for(i = 0;i< dev->outputs_config.num_output;i++)
    {
        struct zxfb_output *output = &dev->outputs_config.outputs[i];
        switch(blank_mode)
        {
        case FB_BLANK_UNBLANK:
            dpms.output = output->type;
            dpms.mode = 0; //on
            krnl_set_dpms(dev->adapter, &dpms);
        case FB_BLANK_NORMAL:
        case FB_BLANK_VSYNC_SUSPEND:
        case FB_BLANK_HSYNC_SUSPEND:
            break;
        case FB_BLANK_POWERDOWN:
            dpms.output = output->type;
            dpms.mode = 4; //off
            krnl_set_dpms(dev->adapter, &dpms);
            break;
        default:
            break;
        }
    }
    return ret;
}

/* pan display */
static int zxfb_pan_display(struct fb_var_screeninfo* var, struct fb_info* info)
{
    struct zxfb_dev    *dev    = info->par;
    struct zxfb_screen *screen = &dev->screen;
    struct zxfb_outputs_config *outputs_config = &dev->outputs_config;
    struct zxfb_output *output = NULL;
    zx_stream_info_t *stream_info  = NULL;
    int i = 0;

    if(var->xoffset + var->xres > var->xres_virtual)
    {
        return -EINVAL;
    }

    if(var->yoffset + var->yres > var->yres_virtual)
    {
        return -EINVAL;
    }

    for(i = 0; i < outputs_config->num_output; i++)
    {
        output = &outputs_config->outputs[i];
        if(!output->connected || (output->crtc == -1))
        {
            continue;
        }

        if(dev->chip_id == CHIP_ELT2K ||
           dev->chip_id == CHIP_E2UMA ||
           dev->chip_id == CHIP_CHX001 ||
           dev->chip_id == CHIP_CHX002)
        {
            ZX_STREAM_TYPE  stream_type = dev->stream_type;
            int crtc_index = output->crtc;
            stream_info  = &(dev->display_frame[crtc_index].stream_info[stream_type]);
            
            if((stream_info->surface != screen->hAllocation)
                ||(stream_info->src_rect.x1 != var->xoffset)
                ||(stream_info->src_rect.y1 != var->yoffset))
            {
                stream_info->surface       = screen->hAllocation;
                stream_info->src_rect.x1   = var->xoffset;
                stream_info->src_rect.y1   = var->yoffset;
                dev->display_frame[crtc_index].hDevice = dev->hDevice;
                dev->display_frame[crtc_index].hContext = dev->hContext;
                dev->display_frame[crtc_index].active_stream_mask   = (1 << stream_type);
                dev->display_frame[crtc_index].acquired_stream_mask = (1 << stream_type);

                zx_info("%s(): iga-%d, stream_type-%d, src[%d,%d, %d,%d], dst[%d,%d, %d,%d]\n", __func__, crtc_index, stream_type,
                    stream_info->src_rect.x1, stream_info->src_rect.y1, stream_info->src_rect.x2, stream_info->src_rect.y2,
                    stream_info->dst_rect.x1, stream_info->dst_rect.y1, stream_info->dst_rect.x2, stream_info->dst_rect.y2);

                krnl_display_frame(dev->adapter, &(dev->display_frame[crtc_index]));
            }
        }
        else
        {
            zx_ps_info_t set_ps  = {0};

            set_ps.x             = var->xoffset;
            set_ps.y             = var->yoffset;
            set_ps.crtc          = output->crtc;
            set_ps.surface = screen->hAllocation;
            krnl_set_ps(dev->adapter, &set_ps);
        }
    }

    return 0;
}

static void zxfb_fillrect(struct fb_info* info, const struct fb_fillrect* rect)
{
    cfb_fillrect(info, rect);
}
static void zxfb_copyarea(struct fb_info* info, const struct fb_copyarea* region)
{
    cfb_copyarea(info, region);
}
static void zxfb_imageblit(struct fb_info* info, const struct fb_image* image)
{
    cfb_imageblit(info, image);
}

struct fb_ops zxfb_ops =
{
    .owner          = THIS_MODULE,
    .fb_open        = zxfb_open,
    .fb_release     = zxfb_release,
    .fb_check_var   = zxfb_check_var,
    .fb_set_par     = zxfb_set_par,
    .fb_setcolreg   = zxfb_setcolreg,
    .fb_blank       = zxfb_blank,
    .fb_pan_display = zxfb_pan_display,
    .fb_fillrect    = zxfb_fillrect,
    .fb_copyarea    = zxfb_copyarea,
    .fb_imageblit   = zxfb_imageblit,
    .fb_ioctl       = NULL,
    .fb_cursor      = NULL,
    .fb_sync        = NULL,
};

